"""Aerotech"""
import time
from concert.quantities import q
from concert.connections.inet import Aerotech
from concert.devices.base import LinearCalibration
from concert.devices.motors.base import ContinuousMotor, Motor


class Aerorot(ContinuousMotor):

    """Aerorot (Continuous Motor) class implementation."""
    HOST = "192.168.18.19"
    PORT = 8001
    AXIS = "X"

    # status constants (bits of the AXISSTATUS output (see HLe docs))
    AXISSTATUS_ENABLED = 0
    AXISSTATUS_HOMED = 1
    AXISSTATUS_IN_POSITION = 2
    AXISSTATUS_MOVE_ACTIVE = 3
    AXISSTATUS_ACCEL_PHASE = 4
    AXISSTATUS_DECEL_PHASE = 5
    AXISSTATUS_POSITION_CAPTURE = 6
    AXISSTATUS_HOMING = 14

    SLEEP_TIME = 0.01

    def __init__(self):
        pos_calib = LinearCalibration(1 / q.deg, 0 * q.deg)
        velo_calib = LinearCalibration(1 / (q.deg / q.sec), 0 * q.deg / q.sec)
        super(Aerorot, self).__init__(pos_calib, velo_calib)

        self["position"].unit = q.deg
        self["velocity"].unit = q.deg / q.sec

        self._connection = Aerotech(Aerorot.HOST, Aerorot.PORT)
        self._connection.execute("ENABLE %s" % (Aerorot.AXIS))

    def __del__(self):
        self._connection.execute("DISABLE %s" % (Aerorot.AXIS))

    def _query_state(self):
        return int(self._connection.execute("AXISSTATUS(%s)" % (Aerorot.AXIS)))

    def _get_position(self):
        return float(self._connection.execute("PFBK(%s)" % (Aerorot.AXIS)))

    def _set_position(self, steps):
        self._connection.execute("MOVEABS %s %f" % (Aerorot.AXIS, steps))

        while not self._query_state() >> Aerorot.AXISSTATUS_IN_POSITION & 1:
            time.sleep(Aerorot.SLEEP_TIME)

    def _get_velocity(self):
        return float(self._connection.execute("VFBK(%s)" % (Aerorot.AXIS)))

    def _set_velocity(self, steps):
        self._connection.execute("FREERUN %s %f" % (Aerorot.AXIS, steps))

        while self._query_state() >> Aerorot.AXISSTATUS_ACCEL_PHASE & 1:
            time.sleep(Aerorot.SLEEP_TIME)

    def _get_state(self):
        res = self._query_state()
        if res >> Aerorot.AXISSTATUS_MOVE_ACTIVE & 1:
            state = Motor.MOVING
        elif res >> Aerorot.AXISSTATUS_IN_POSITION & 1:
            state = Motor.STANDBY
        else:
            state = Motor.NA

        return state

    def _stop(self):
        if self.state == Motor.MOVING:
            self._connection.execute("ABORT %s" % (Aerorot.AXIS))

        while self.state == Motor.MOVING:
            time.sleep(Aerorot.SLEEP_TIME)

    def _home(self):
        self._connection.execute("HOME %s" % (Aerorot.AXIS))

        while self.state == Motor.MOVING:
            time.sleep(Aerorot.SLEEP_TIME)

    def get_digital_in(self, port, bit):
        """Get TTL level on *port* and *bit*."""
        res = self._connection.execute("DIN(%s,%d,%d)" %
                                       (Aerorot.AXIS, port, bit))
        return res == "1"

    def set_digital_out(self, port, bit, value):
        """Set TTL level on *port* and *bit* to *value*"""
        self._connection.execute("DOUT %s, %d, %d:%d" % (Aerorot.AXIS, port,
                                                         bit, value))



"""
A :class:`Camera` can be set via the device-specific properties that can be set
and read with :meth:`.Parameter.set` and :meth:`.Parameter.get`.  Moreover, a
camera provides means to

* :meth:`~Camera.start_recording` frames,
* :meth:`~Camera.stop_recording` the acquisition,
* :meth:`~Camera.trigger` a frame capture and
* :meth:`~Camera.grab` to get the last frame.

To setup and use a camera in a typical environment, you would do::

    import numpy as np
    from concert.devices.cameras.uca import UcaCamera

    camera = UcaCamera('pco')
    camera.exposure_time = 0.2 * q.s
    camera.start_recording().wait()
    camera.trigger().wait()
    data = camera.grab()
    camera.stop_recording()

    print("mean=%f, stddev=%f" % (np.mean(data), np.std(data))
"""
from concert.devices.base import Device


class Camera(Device):

    """Base class for remotely controllable cameras."""

    def __init__(self, params):
        super(Camera, self).__init__(params)

    def start_recording(self):
        """Start recording frames."""
        self._record_real()

    def stop_recording(self):
        """Stop recording frames."""
        self._stop_real()

    def trigger(self):
        """Trigger a frame if possible."""
        self._trigger_real()

    def grab(self):
        """Return a NumPy array with data of the current frame."""
        return self._grab_real()

    def _record_real(self):
        raise NotImplementedError

    def _stop_real(self):
        raise NotImplementedError

    def _trigger_real(self):
        raise NotImplementedError

    def _grab_real(self):
        raise NotImplementedError



import unittest
import logbook
from testfixtures import ShouldRaise, compare
from concert.base import Parameter, ParameterError
from concert.devices.base import Device
from concert.ui import get_default_table


class MockDevice(Device):

    def __init__(self):
        def setter(value):
            pass

        def getter():
            return 1

        self.params = [Parameter('readonly', fget=getter),
                       Parameter('writeonly', fset=setter)]

        super(MockDevice, self).__init__(self.params)


class TestDevice(unittest.TestCase):

    def setUp(self):
        self.device = MockDevice()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_accessor_functions(self):
        compare(self.device.get_readonly().result(), 1)
        self.device.set_writeonly(None).wait()

    def test_iterable(self):
        for param in self.device:
            self.assertTrue(param.name in ('readonly', 'writeonly', 'state'))

    def test_get_parameter(self):
        compare(self.device['readonly'], self.device.params[0])
        compare(self.device['writeonly'], self.device.params[1])

    def test_invalid_paramter(self):
        with ShouldRaise(ParameterError):
            param = self.device['foo']
            compare(param, None)

    def test_str(self):
        table = get_default_table(["Parameter", "Value"])
        table.border = False
        table.add_row(["readonly", "1"])
        table.add_row(["state", Device.NA])

        compare(str(self.device), table.get_string())

    def test_context_manager(self):
        # This is just a functional test, we don't cover synchronization issues
        # here.
        with self.device as d:
            v = d.readonly
            d.writeonly = 2



"""
A device is an abstraction for a piece of hardware that can be controlled.

The main interface to all devices is a generic setter and getter mechanism.
:meth:`Device.set` sets a parameter to value. Additionally, you can specify a
*blocking* parameter to halt execution until the value is actually set on the
device::

    axis.set('position', 5.5 * q.mm, blocking=True)

    # This will be set once axis.set() has finished
    camera.set('exposure-time', 12.2 * q.s)

Some devices will provide convenience accessor methods. For example, to set the
position on an axis, you can also use :meth:`.Axis.set_position`.

:meth:`Device.get` simply returns the current value.
"""
import threading
from logbook import Logger
from concert.base import Parameterizable, Parameter
from concert.quantities import q


LOG = Logger(__name__)


class Device(Parameterizable):

    """
    A :class:`Device` provides locked access to a real-world device and
    provides a :attr:`state` :class:`.Parameter`.

    A implements the context protocol to provide locking and can be used like
    this ::

        with device:
            # device is locked
            device.parameter = 1 * q.m
            ...

        # device is unlocked again

    .. py:attribute:: state

        Current state of the device.
    """

    NA = "n/a"

    def __init__(self, parameters=None):
        super(Device, self).__init__(parameters)
        self.add_parameter(Parameter('state', self._get_state,
                                     owner_only=True))
        self._lock = threading.Lock()
        self._states = set([self.NA])
        self._state = self.NA

    def __enter__(self):
        self._lock.acquire()
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._lock.release()

    def _get_state(self):
        return self._state

    def _set_state(self, state):
        if state in self._states:
            self._state = state
            self['state'].notify()
        else:
            LOG.warn("State {0} unknown.".format(state))


class Calibration(object):

    """Interface to convert between user and device units."""

    def to_user(self, value):
        """Return *value* in user units."""
        raise NotImplementedError

    def to_steps(self, value):
        """Return *value* in device units."""
        raise NotImplementedError


class LinearCalibration(Calibration):

    """A linear calibration maps a number of steps to a real-world unit.

    *steps_per_unit* tells how many steps correspond to some unit,
    *offset_in_steps* by how many steps the device is away from some zero
    point.
    """

    def __init__(self, steps_per_unit, offset_in_steps):
        super(LinearCalibration, self).__init__()
        self._steps_per_unit = steps_per_unit
        self._offset = offset_in_steps

    def to_user(self, value_in_steps):
        return value_in_steps * q.count / self._steps_per_unit - self._offset

    def to_steps(self, value):
        res = (value + self._offset) * self._steps_per_unit
        return res.to_base_units().magnitude



import unittest
import logbook
from concert.quantities import q
from concert.tests import assert_almost_equal
from concert.devices.motors.dummy import Motor
from concert.optimization import algorithms as algs
from concert.optimization.optimizers import Minimizer, Maximizer
from concert.tests import slow


class TestOptimizers(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()
        self.algorithms = [algs.halver, algs.down_hill, algs.powell,
                           algs.nonlinear_conjugate, algs.bfgs,
                           algs.least_squares]
        self.center = 3.0 * q.mm
        self.motor = Motor(position=0)

    def tearDown(self):
        self.handler.pop_application()

    def feedback(self):
        return (self.motor.position.to_base_units().magnitude -
                self.center.to_base_units().magnitude) ** 2

    def check(self):
        assert_almost_equal(self.motor.position.to_base_units(),
                            self.center.to_base_units(), 1e-2)

    @slow
    def test_minimizers(self):
        for i in range(len(self.algorithms)):
            self.motor.position = 0 * q.mm
            minim = Minimizer(self.motor["position"], self.feedback,
                              self.algorithms[i])
            minim.run().wait()
            self.check()

    @slow
    def test_maximizers(self):
        for i in range(len(self.algorithms)):
            self.motor.position = 0 * q.mm
            minim = Maximizer(self.motor["position"],
                              lambda: - self.feedback(), self.algorithms[i])
            minim.run().wait()
            self.check()



"""The :mod:`.asynchronous` module provides mechanisms for asynchronous
execution and messaging.
"""
try:
    import Queue as queue
except ImportError:
    import queue

import threading
from concurrent.futures import ThreadPoolExecutor, Future
from functools import wraps


# Patch futures so that they provide a wait() method
def _wait(self, _timeout=None):
    self.result()
    return self

Future.wait = _wait

# Module-wide executor
EXECUTOR = ThreadPoolExecutor(max_workers=128)

# Module-wide disable
DISABLE = False


class _FakeFuture(Future):

    def __init__(self, result):
        super(_FakeFuture, self).__init__()
        self.set_result(result)


def async(func):
    """A decorator for functions which are supposed to be executed
    asynchronously."""

    if DISABLE:
        @wraps(func)
        def _sync(*args, **kwargs):
            result = func(*args, **kwargs)
            return _FakeFuture(result)

        _sync.__dict__["_async"] = True
        return _sync
    else:
        @wraps(func)
        def _async(*args, **kwargs):
            return EXECUTOR.submit(func, *args, **kwargs)

        _async.__dict__["_async"] = True
        return _async


def is_async(func):
    """Returns *True* if the given function *func* is asynchronous."""
    return hasattr(func, "_async") and getattr(func, "_async")


def wait(futures):
    """Wait for the list of *futures* to finish and raise exceptions if
    happened."""
    for future in futures:
        future.result()


class Dispatcher(object):

    """Core dispatcher"""

    def __init__(self):
        self._subscribers = {}
        self._messages = queue.Queue()
        self._event_queues = {}
        self._lock = threading.Lock()

        server = threading.Thread(target=self._serve)
        server.daemon = True
        server.start()

    def subscribe(self, sender, message, handler):
        """Subscribe to a message sent by sender.

        When message is sent by sender, handler is called with sender as the
        only argument.

        """
        tsm = sender, message
        if tsm in self._subscribers:
            self._subscribers[tsm].add(handler)
        else:
            self._subscribers[tsm] = set([handler])

    def unsubscribe(self, sender, message, handler):
        """Remove *handler* from the subscribers to *(sender, message)*."""
        tsm = sender, message
        if tsm in self._subscribers:
            self._subscribers[tsm].remove(handler)

    def send(self, sender, message):
        """Send message from sender."""
        self._messages.put((sender, message))

    def _serve(self):
        while True:
            tsm = self._messages.get()
            sender, message = tsm

            if tsm in self._subscribers:
                for callback in self._subscribers[tsm]:
                    callback(sender)

            if tsm in self._event_queues:
                self._event_queues[tsm].notify_and_clear()

            self._messages.task_done()


dispatcher = Dispatcher()



import unittest
import logbook
from testfixtures import ShouldRaise
from concert.devices.io.dummy import IO
from concert.base import ReadAccessError, WriteAccessError


class TestDummyIO(unittest.TestCase):

    def setUp(self):
        self.io_device = IO()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_read_port(self):
        self.assertEqual(self.io_device.exposure, 0)
        self.assertEqual(self.io_device.busy, 1)

    def test_write_port(self):
        self.io_device.acq_enable = 1
        self.assertEqual(self.io_device._ports
                         [self.io_device["acq_enable"].port_id], 1)

    def test_read_not_readable_port(self):
        with ShouldRaise(ReadAccessError):
            self.io_device["acq_enable"].get().wait()

    def test_write_not_writable_port(self):
        with ShouldRaise(WriteAccessError):
            self.io_device["busy"].set(1).wait()



"""
Optimization (minimization, maximization) can be done by many techniques.
This module consists of algorithms capable of optimizing functions y = f(x).
"""

from concert.quantities import q


def halver(function, x_0, initial_step=None, epsilon=None,
           max_iterations=100):
    """
    Halving the interval, evaluate *function* based on *param*. Use
    *initial_step*, *epsilon* precision and *max_iterations*.
    """
    # Safe copy for not changing the original.
    if initial_step is None:
        step = x_0 / x_0.magnitude if x_0.magnitude else x_0
    else:
        step = initial_step
    if epsilon is None:
        epsilon = 1e-3 * x_0

        if x_0.magnitude:
            epsilon /= x_0.magnitude

    direction = 1
    i = 0
    last_x = x_0

    y_0 = function(x_0)

    def turn(direction, step):
        return -direction, step / 2.0

    def move(x_0, direction, step):
        return x_0 + direction * step

    x_0 = move(x_0, direction, step)

    while i < max_iterations:
        y_1 = function(x_0)

        if step < epsilon:
            break

        if y_1 >= y_0:
            # Worse, change direction and move to the half of the last
            # good x and the new x.
            direction, step = turn(direction, step)
            x_0 = (x_0 + last_x) / 2
        else:
            # OK, move forward.
            last_x = x_0
            x_0 = move(x_0, direction, step)

        y_0 = y_1
        i += 1

    return x_0


def quantized(function):
    """
    A helper function meant to be used as a decorator to quantize
    a *function* which does not take units into account.
    """
    def wrapper(eval_func, x_0, *args, **kwargs):
        return q.Quantity(function(lambda x:
                                   eval_func(q.Quantity(x, x_0.units)),
                                   x_0.magnitude, *args, **kwargs), x_0.units)

    wrapper.__doc__ = function.__doc__

    return wrapper


@quantized
def down_hill(function, x_0, **kwargs):
    """
    Downhill simplex algorithm from :py:func:`scipy.optimize.fmin`.
    Please refer to the scipy function for additional arguments information.
    """
    from scipy import optimize

    return optimize.fmin(function, x_0, disp=0, **kwargs)[0]


@quantized
def powell(function, x_0, **kwargs):
    """
    Powell's algorithm from :py:func:`scipy.optimize.fmin_powell`.
    Please refer to the scipy function for additional arguments information.
    """
    from scipy import optimize

    return optimize.fmin_powell(function, x_0, disp=0, **kwargs)


@quantized
def nonlinear_conjugate(function, x_0, **kwargs):
    """
    Nonlinear conjugate gradient algorithm from
    :py:func:`scipy.optimize.fmin_cg`.
    Please refer to the scipy function for additional arguments information.
    """
    from scipy import optimize

    return optimize.fmin_cg(function, x_0, disp=0, **kwargs)[0]


@quantized
def bfgs(function, x_0, **kwargs):
    """
    Broyde-Fletcher-Goldfarb-Shanno (BFGS) algorithm from
    :py:func:`scipy.optimize.fmin_bfgs`.
    Please refer to the scipy function for additional arguments information.
    """
    from scipy import optimize

    return optimize.fmin_bfgs(function, x_0, disp=0, **kwargs)[0]


@quantized
def least_squares(function, x_0, **kwargs):
    """
    Least squares algorithm from :py:func:`scipy.optimize.leastsq`.
    Please refer to the scipy function for additional arguments information.
    """
    from scipy import optimize

    return optimize.leastsq(function, x_0, **kwargs)[0][0]



"""
Optimization is a procedure to iteratively find the best possible match
to

.. math::
    y = f(x).

This module provides base classes for optimizer implementations.
"""

import logbook
from concert.processes.base import Process
from concert.asynchronous import async, dispatcher
from concert.base import LimitError


LOG = logbook.Logger(__name__)


class Optimizer(Process):

    """The most abstract optimizer."""
    FINISHED = "optimization-finished"

    def __init__(self, function):
        """
        Create an optimizer for a *function*.
        """
        super(Optimizer, self).__init__(None)
        self.data = []
        self.function = function

    def evaluate(self, x):
        """Execute y = f(*x*), save (x, y) pair and return y."""
        y = self.function(x)
        self.data.append((x, y))

        return y

    def _optimize(self):
        """Optimization executive code."""
        raise NotImplementedError

    @async
    def run(self):
        """
        run()

        Run the optimizer.
        """
        # Clear the saved values.
        self.data = []

        self._optimize()

        dispatcher.send(self, self.FINISHED)
        LOG.debug("Optimization finished with x = {0}, y = {1}".
                  format(self.data[-1][0], self.data[-1][1]))

        return tuple(self.data)


class ParameterOptimizer(Optimizer):

    """
    An optimizer based on a :py:class:`.Parameter` and a callable feedback.
    The function to optimize is created as ::

        def function(x):
            param.set(x).wait()
            return feedback()
    """

    def __init__(self, param, feedback):
        """Create an optimizer for parameter *param* and *feedback*."""
        self.param = param
        self.feedback = feedback

        def function(x):
            try:
                self.param.set(x).wait()
            except LimitError:
                pass

            return self.feedback()

        super(ParameterOptimizer, self).__init__(function)



"""Tk Widget."""
import Queue
import Tkinter as tk
from concert.asynchronous import dispatcher


class DeviceWidget(tk.Toplevel):

    """Wrap a device in a top level Tk widget.

    *device* must be an object derived from :class:`.Device`. *timeout*
    specifies how much time in milliseconds passes between to parameter
    queries.
    """

    def __init__(self, device, timeout=1500):
        tk.Toplevel.__init__(self, padx=6, pady=6)

        self.title(device.__class__.__name__)
        self.grid()
        self.queues = {}
        self.values = {}
        self.timeout = timeout

        grid_opts = {'padx': 3, 'pady': 3}

        for i, param in enumerate(device):
            name_label = tk.Label(self, text=param.name)
            name_label.grid(row=i, column=0, **grid_opts)

            if param.is_readable():
                value = tk.DoubleVar()
                value.set(param.get().result())

                state = tk.NORMAL if param.is_writable() else 'readonly'
                entry = tk.Entry(self, textvariable=value, state=state)
                entry.grid(row=i, column=1, **grid_opts)

                self.queues[param] = Queue.Queue()
                self.values[param] = value

                dispatcher.subscribe(param, 'changed', self._update_value)
                self.after(self.timeout, self._poll, param)

            if param.unit:
                unit_name = param.unit.dimensionality.string
                unit_label = tk.Label(self, text=unit_name)
                unit_label.grid(row=i, column=2, **grid_opts)

    def _update_value(self, param):
        self.queues[param].put(param.get().result())

    def _poll(self, param):
        try:
            result = self.queues[param].get(False)
            self.values[param].set(result)
        except Queue.Empty:
            pass

        self.after(self.timeout, self._poll, param)



import logging
import os
import nose.plugins


LOG = logging.getLogger(__name__)


class DisableAsync(nose.plugins.Plugin):
    name = 'disable_async'

    def options(self, parser, env=os.environ):
        parser.add_option('--disable-async', action='store_true',
                          default=False, dest='disable_async',
                          help="Disable asynchronous execution.")
        super(DisableAsync, self).options(parser, env=env)

    def configure(self, options, conf):
        if options.disable_async:
            import concert.asynchronous
            concert.asynchronous.DISABLE = True

        super(DisableAsync, self).configure(options, conf)



import unittest
import logbook
import random
from testfixtures import ShouldRaise
from concert.tests import assert_almost_equal
from concert.quantities import q
from concert.devices.base import LinearCalibration
from concert.devices.monochromators.dummy import\
    Monochromator as DummyMonochromator
from concert.devices.monochromators import base
from concert.devices.monochromators.base import Monochromator


class WavelengthMonochromator(Monochromator):

    """A monochromator which implements wavelength getter and setter. The
    conversion needs to be handled in the base class."""

    def __init__(self):
        super(WavelengthMonochromator, self).__init__(self)
        self._wavelength = random.random() * 1e-10 * q.m

    def _get_wavelength(self):
        return self._wavelength

    def _set_wavelength(self, wavelength):
        self._wavelength = wavelength


class UselessMonochromator(Monochromator):

    """A monochromator wihch does not implement anything."""

    def __init__(self):
        super(UselessMonochromator, self).__init__(self)


class TestDummyMonochromator(unittest.TestCase):

    def setUp(self):
        calibration = LinearCalibration(1 * q.eV, 0 * q.eV)
        self.mono = DummyMonochromator(calibration)
        self.wave_mono = WavelengthMonochromator()
        self.useless_mono = UselessMonochromator()
        self.energy = 25 * q.keV
        self.wavelength = 0.1 * q.nm
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_useless_mono_energy(self):
        with ShouldRaise(NotImplementedError):
            self.useless_mono.energy
        with ShouldRaise(NotImplementedError):
            self.useless_mono.energy = 25 * q.keV

    def test_useless_mono_wavelength(self):
        with ShouldRaise(NotImplementedError):
            self.useless_mono.wavelength
        with ShouldRaise(NotImplementedError):
            self.useless_mono.wavelength = 1e-10 * q.m

    def test_energy_mono_energy(self):
        self.mono.energy = self.energy
        assert_almost_equal(self.mono.energy, self.energy)
        assert_almost_equal(self.mono.wavelength,
                            base.energy_to_wavelength(self.mono.energy))

    def test_energy_mono_wavelength(self):
        self.mono.wavelength = self.wavelength
        assert_almost_equal(self.mono.wavelength, self.wavelength)
        assert_almost_equal(base.wavelength_to_energy(self.wavelength),
                            self.mono.energy)

    def test_wavelength_mono_energy(self):
        self.wave_mono.energy = self.energy
        assert_almost_equal(self.wave_mono.energy, self.energy)
        assert_almost_equal(self.wave_mono.wavelength,
                            base.energy_to_wavelength(self.wave_mono.energy))

    def test_wavelength_mono_wavelength(self):
        # Wavelength-based monochromator.
        self.wave_mono.wavelength = self.wavelength
        assert_almost_equal(self.wave_mono.wavelength, self.wavelength)
        assert_almost_equal(base.wavelength_to_energy(self.wavelength),
                            self.wave_mono.energy)



"""Motor Controller"""
from concert.devices.base import Device
from concert.base import Parameter


class MotorController(Device):

    """Base class for motor controllers."""

    def __init__(self):
        params = [Parameter("motors", self._get_motors)]
        super(MotorController, self).__init__(params)

    def _get_motors(self):
        raise NotImplementedError



"""Port, IO Device."""
from concert.devices.base import Device
from concert.base import Parameter


class Port(Parameter):

    """Port is a :class:`.Parameter` with an id. It represents a low-level
    port capable of sending or receiving electrical signals. *port_id* is the
    identification of the port within a device. When the device wants to
    access this port it will refer to it by the *port_id*."""

    def __init__(self, port_id, name, fget=None, fset=None, doc=None):
        if fget is None:
            new_fget = None
        else:
            new_fget = lambda: fget(port_id)
        if fset is None:
            new_fset = None
        else:
            new_fset = lambda value: fset(port_id, value)
        Parameter.__init__(self, name, fget=new_fget, fset=new_fset, doc=doc)
        self.port_id = port_id


class IO(Device):

    """The IO device consists of ports which can be readable, writable or
    both.
    """

    def __init__(self, ports=None):
        super(IO, self).__init__(ports)

    def read_port(self, port):
        """Read a *port*."""
        raise NotImplementedError

    def write_port(self, port, value):
        """Write a *value* to a *port*."""
        raise NotImplementedError



"""
The :mod:`.alignment` module provides pre-defined functions built from the
optimization classes to execute common alignment procedures.
"""
import numpy as np
from concert.quantities import q
from concert.optimization.algorithms import halver
from concert.optimization.optimizers import Maximizer


def focus(camera, motor, measure=np.std):
    """
    Focus *camera* by moving *motor*. *measure* is a callable that computes a
    scalar that has to be maximized from an image taken with *camera*.

    This function is returning a future encapsulating the focusing event. Note,
    that the camera is stopped from recording as soon as the optimal position
    is found.
    """
    opts = {'initial_step': 10 * q.mm, # we should guess this from motor limits
            'epsilon': 5e-3 * q.mm}

    def get_measure():
        frame = camera.grab()
        return measure(frame)

    maximizer = Maximizer(motor['position'],
                          get_measure,
                          halver, alg_kwargs=opts)

    camera.start_recording()
    f = maximizer.run()
    f.add_done_callback(lambda: camera.stop_recording())
    return f



"""
NeXus file format related utitilities.

This module provides convenience utilities to create NeXus-compatible data
sets. It uses NeXpy_ to interface with NeXus.

.. _NeXpy: http://wiki.nexusformat.org/NeXpy
"""
import numpy as np
from nexpy.api import nexus as nx
from concert.base import Parameter
from concert.devices import cameras, monochromators
from concert.asynchronous import async


def get_detector(_camera):
    """Return an nx.NXdetector instance of *camera*."""
    return nx.NXdetector()


def get_monochromator(monochromator):
    """Return an nx.NXmonochromator instance of *monochromator*."""
    return nx.NXmonochromator(energy=monochromator.energy,
                              wavelength=monochromator.wavelength)


def get_instrument(device):
    """Tries to return a corresponding nx.NXinstrument for *device* or
    *None*."""
    type_map = {
        cameras.base.Camera: get_detector,
        monochromators.base.Monochromator: get_monochromator
    }

    for tval, convert in type_map.items():
        if isinstance(device, tval):
            return convert(device)

    return None


def quantity_unit(quantity):
    """Return string representation of *quantity*'s unit."""
    return str(quantity.dimensionality)


def add_quantity(nx_object, name, quantity):
    """Add *quantity* value as *name* to *nx_object*.

    The magnitude of *quantity* is used as a value and the string
    representation of the dimensionality of *quantity* is added to the field as
    *units* ::

        sample = nx.NXdetector()
        add_quantity(sample, 'x_pixel_size', 5 * q.micrometer)

        assert sample.x_pixel_size == 5
        assert sample.x_pixel_size.units = 'um'
    """
    setattr(nx_object, name, quantity.magnitude)
    attr = getattr(nx_object, name)
    setattr(attr, 'units', quantity_unit(quantity))


@async
def get_scan_result(scanner):
    """Create an nx.NXdata tuple from a scan.

    Runs *scanner* and turns the result into two nx.SDSs and returns an
    nx.NXdata tuple from both SDSs.
    """
    x_raw, y_raw = scanner.run().result()
    x_nx = nx.SDS(x_raw,
                  name=scanner.param.name,
                  units=quantity_unit(scanner.minimum))

    if isinstance(scanner.feedback, Parameter):
        y_nx = nx.SDS(y_raw,
                      name=scanner.feedback.name,
                      units=quantity_unit(scanner.feedback.unit))
    else:
        y_nx = nx.SDS(y_raw)

    return nx.NXdata(y_nx, [x_nx])


@async
def get_tomo_scan_result(tomo_scanner, title='Tomo scan'):
    """Create and return a NXtomophase data set from the scan results of
    *tomo_scanner*."""
    future = tomo_scanner.run()
    detector = tomo_scanner.camera

    root = nx.NXentry(title=title)

    n_projections = tomo_scanner.num_projections
    step = tomo_scanner.angle.magnitude
    angles = np.arange(n_projections * step, step=step)
    zeros = np.zeros(n_projections)
    sample = nx.NXsample(rotation_angle=angles,
                         x_translation=zeros,
                         y_translation=zeros,
                         z_translation=zeros)

    sample.rotation_angle.units = quantity_unit(tomo_scanner.angle)
    sample.x_translation.units = "m"
    sample.y_translation.units = "m"
    sample.z_translation.units = "m"

    root.sample = sample
    root.instrument = nx.NXinstrument()

    darks, flats, projections = future.result()

    dark_field = nx.NXdetector(sequence_number=len(darks),
                               data=darks)
    bright_field = nx.NXdetector(sequence_number=len(flats),
                                 data=flats)
    sample = nx.NXdetector(sequence_number=len(projections),
                           data=projections)

    root.instrument.dark_field = dark_field
    root.instrument.bright_field = bright_field
    root.instrument.sample = sample

    add_quantity(sample, 'x_pixel_size', detector.sensor_pixel_width)
    add_quantity(sample, 'y_pixel_size', detector.sensor_pixel_height)

    counts = [np.sum(frame) for frame in darks]
    counts.extend([np.sum(frame) for frame in flats])
    counts.extend([np.sum(frame) for frame in projections])

    control = nx.NXmonitor(integral=counts)
    root.control = control

    return root



"""
The :mod:`.scan` module provides process classes and functions to change
parameters along pre-computed "trajectories".

:class:`Scanner` objects scan a parameter and evaluate a dependent variable ::

    from concert.processes.base import Scanner

    motor = Motor()
    camera = Camera()
    scanner = Scanner(motor['position'], lambda: camera.grab())
    scanner.minimum = 0 * q.mm
    scanner.maximum = 2 * q.mm

    x, y = scanner.run().result()

As you can see, the position of *motor* is varied and a frame at each interval
point is taken. Because processes run asynchronously, we call :meth:`result` on
the future that is returned by :meth:`~.Scanner.run`. This yields a tuple
with x and y values, corresponding to positions and frames.

For some tasks, feedbacks (such as the frame grabbing in the example above) are
pre-defined in the :mod:`concert.feedbacks.camera` module.

:func:`ascan` and :func:`dscan` are used to scan multiple parameters
in a similar way as SPEC::

    from concert.quantities import q
    from concert.processes.scan import ascan

    def do_something(parameters):
        for each parameter in parameters:
            print(parameter)

    ascan([(motor1['position'], 0 * q.mm, 25 * q.mm),
           (motor2['position'], -2 * q.cm, 4 * q.cm)],
           n_intervals=10, handler=do_something)

"""
import numpy as np
from concert.quantities import q
from concurrent.futures import wait
from concert.base import Parameter
from concert.asynchronous import async
from concert.processes.base import Process


class Scanner(Process):

    """A scan process.

    :meth:`.Scanner.run` sets *param* to :attr:`.intervals` data points and
    calls `feedback()` on each data point.

    .. py:attribute:: param

        The scanned :class:`.Parameter`. It must have the same unit as
        :attr:`minimum` and :attr:`maximum`.

    .. py:attribute:: feedback

        A callable that must return a scalar value. It is called for each data
        point.

    .. py:attribute:: minimum

        The lower bound of the scannable range.

    .. py:attribute:: maximum

        The upper bound of the scannable range.

    .. py:attribute:: intervals

        The number of intervals that are scanned.
    """

    def __init__(self, param, feedback):
        params = [Parameter('minimum', doc="Left bound of the interval"),
                  Parameter('maximum', doc="Right bound of the interval"),
                  Parameter('intervals', doc="Number of intervals")]

        super(Scanner, self).__init__(params)
        self.intervals = 64
        self.param = param
        self.feedback = feedback

    @async
    def run(self, convert=lambda x: x):
        """run()

        Set :attr:`param` to values between :attr:`minimum` and
        :attr:`maximum`, call :attr:`feedback` on each data point and return a
        future with the value tuple.

        The result tuple *(x, y)* contains two array-likes with the same shape.
        *x* contains the values that :attr:`param` has taken during the scan
        whereas *y* contains the values evaluated by :attr:`feedback`.
        """
        xss = np.linspace(self.minimum, self.maximum, self.intervals)
        yss = []

        for xval in xss:
            self.param.set(convert(xval)).wait()
            yss.append(self.feedback())

        return (xss, yss)

    def show(self):
        """Call :meth:`run`, show the result of the scan with Matplotlib and
        return the plot object."""
        import matplotlib.pyplot as plt
        xval, yval = self.run().result()
        plt.xlabel(self.param.name)
        return plt.plot(xval, yval)


class StepTomoScanner(Process):

    """Tomo Scan Process."""

    def __init__(self, camera, rotary_stage,
                 prepare_dark_scan,
                 prepare_flat_scan,
                 prepare_proj_scan):

        params = [Parameter('angle', unit=q.deg)]

        self.camera = camera
        self.rotary_stage = rotary_stage
        self.prepare_dark_scan = prepare_dark_scan
        self.prepare_flat_scan = prepare_flat_scan
        self.prepare_proj_scan = prepare_proj_scan
        self.num_projections = 4

        super(StepTomoScanner, self).__init__(params)

    @async
    def run(self):
        def take_frames(prepare_step, n_frames=2):
            """Take frames."""
            frames = []
            prepare_step()
            self.camera.start_recording()

            for _ in xrange(n_frames):
                frames.append(self.camera.grab())

            self.camera.stop_recording()
            return frames

        darks = take_frames(self.prepare_dark_scan)
        flats = take_frames(self.prepare_flat_scan)

        projections = []
        step = self.angle

        self.prepare_proj_scan()
        self.camera.start_recording()

        for _ in xrange(self.num_projections):
            self.rotary_stage.move(step)
            projections.append(self.camera.grab())

        self.camera.stop_recording()
        return darks, flats, projections


def _pull_first(tuple_list):
    for tup in tuple_list:
        yield tup[0]


def ascan(param_list, n_intervals, handler, initial_values=None):
    """
    For each of the *n_intervals* and for each of the *(parameter, start,
    stop)* tuples in *param_list*, calculate a set value from *(stop - start) /
    n_intervals* and set *parameter* to it::

        ascan([(motor['position'], 0 * q.mm, 2 * q.mm)], 5, handler)

    When all devices have reached the set point *handler* is called with a list
    of the parameters as its first argument.

    If *initial_values* is given, it must be a list with the same length as
    *devices* containing start values from where each device is scanned.
    """
    parameters = [param for param in _pull_first(param_list)]

    if initial_values:
        if len(param_list) != len(initial_values):
            raise ValueError("*initial_values* must match *parameter_list*")
    else:
        initial_values = []

        for param in parameters:
            if param.unit:
                initial_values.append(0 * param.unit)
            else:
                initial_values.append(0)

    initialized_params = map(lambda (tup, single): tup + (single,),
                             zip(param_list, initial_values))

    for i in range(n_intervals + 1):
        futures = []

        for param, start, stop, init in initialized_params:
            step = (stop - start) / n_intervals
            value = init + start + i * step
            futures.append(param.set(value))

        wait(futures)
        handler(parameters)


def dscan(parameter_list, n_intervals, handler):
    """
    For each of the *n_intervals* and for each of the *(parameter, start,
    stop)* tuples in *param_list*, calculate a set value from *(stop - start) /
    n_intervals* and set *parameter*.
    """
    initial_values = [param.get().result()
                      for param in _pull_first(parameter_list)]

    return ascan(parameter_list, n_intervals, handler, initial_values)



import unittest
import logbook
import numpy as np
from threading import Event
from testfixtures import ShouldRaise
from concert.quantities import q
from concert.devices.motors.dummy import Motor
from concert.devices.base import LinearCalibration
from concert.asynchronous import dispatcher
from concert.tests import slow
from concert.processes.tomoalignment import Aligner
from concert.measures.rotationaxis import Ellipse
from concert.tests.util.rotationaxis import SimulationCamera
from concert.processes.scan import Scanner


class TestDummyAlignment(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()
        self.x_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.x_motor["position"].unit = q.deg
        self.y_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.y_motor["position"].unit = q.deg
        self.z_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.z_motor["position"].unit = q.deg

        self.x_motor.position = 0 * q.deg
        self.z_motor.position = 0 * q.deg

        self.image_source = SimulationCamera(256, self.x_motor["position"],
                                             self.y_motor["position"],
                                             self.z_motor["position"])

        # A scanner which scans the rotation axis.
        self.scanner = Scanner(self.y_motor["position"],
                               self.image_source.grab)
        self.scanner.minimum = 0 * q.rad
        self.scanner.maximum = 2 * np.pi * q.rad
        self.scanner.intervals = 10

        dispatcher.subscribe(self.y_motor["position"],
                             self.y_motor["position"].CHANGED,
                             self.iteration_listener)
        self.aligner = Aligner(Ellipse(), self.scanner,
                               self.x_motor, self.z_motor)
        dispatcher.subscribe(self.aligner, self.aligner.AXIS_ALIGNED,
                             self.alignment_listener)

        self.iteration = 0
        self.max_iterations = 10

        # Alignment finishes after the aligner finishes or it iterates
        # too much, in which case the test fails.
        self.alignment_finished = Event()

        # Allow 1 px misalignment in y-direction.
        self.eps = np.arctan(2.0 / self.image_source.rotation_radius) * q.rad

    def tearDown(self):
        self.handler.pop_application()

    def iteration_listener(self, sender):
        self.iteration += 1

        if self.iteration / self.scanner.intervals == \
                self.max_iterations:
            self.alignment_finished.set()

    def alignment_listener(self, aligner):
        self.alignment_finished.set()

    def check_alignment(self):
        """Wait either for aligner to finish or for maximum iterations number
        reached.
        """
        self.alignment_finished.wait()

        self.assertTrue(self.max_iterations > self.image_source.iteration,
                        "Maximum iterations exceeded.")

    def align_check(self, x_angle, z_angle, has_z_motor=True):
        """"Align and check th eresults."""
        self.x_motor.position = z_angle
        self.z_motor.position = x_angle

        self.aligner.z_motor = self.z_motor if has_z_motor else None
        self.aligner.run()

        self.check_alignment()

        # In our case the best perfectly aligned position is when both
        # motors are in 0.
        assert np.abs(self.x_motor.position) < self.eps
        if has_z_motor:
            assert np.abs(self.z_motor.position) < self.eps

    @slow
    def test_out_of_fov(self):
        def get_ones():
            return np.ones((self.image_source.size,
                            self.image_source.size))

        self.scanner.feedback = get_ones
        with ShouldRaise(ValueError("No sample tip points found.")):
            self.aligner.run().wait()

    @slow
    def test_not_offcentered(self):
        self.image_source.rotation_radius = 0
        with ShouldRaise(ValueError("Sample off-centering too " +
                                    "small, enlarge rotation radius.")):
            self.aligner.run().wait()

    @slow
    def test_no_x_axis(self):
        """Test the case when there is no x-axis motor available."""
        self.align_check(17 * q.deg, 11 * q.deg, has_z_motor=False)

    @slow
    def test_not_misaligned(self):
        "Perfectly aligned rotation axis."
        self.align_check(0 * q.deg, 0 * q.deg)

    @slow
    def test_only_x(self):
        """Only misaligned laterally."""
        self.align_check(-17 * q.deg, 0 * q.deg)

    @slow
    def test_only_z(self):
        """Only misaligned in the beam direction."""
        self.align_check(0 * q.deg, 11 * q.deg)

    @slow
    def test_huge_x(self):
        self.image_source.scale = (3, 0.25, 3)
        self.align_check(60 * q.deg, 11 * q.deg)

    @slow
    def test_huge_z(self):
        self.image_source.scale = (3, 0.25, 3)
        self.align_check(11 * q.deg, 60 * q.deg)

    @slow
    def test_positive(self):
        self.align_check(17 * q.deg, 11 * q.deg)

    @slow
    def test_negative_positive(self):
        self.align_check(-17 * q.deg, 11 * q.deg)

    @slow
    def test_positive_negative(self):
        self.align_check(17 * q.deg, -11 * q.deg)

    @slow
    def test_negative(self):
        self.align_check(-17 * q.deg, -11 * q.deg)



"""Connection."""
import logbook
import socket


LOG = logbook.Logger(__name__)


class Connection(object):

    """A two-way socket connection."""

    def __init__(self, host, port):
        self._peer = (host, port)
        self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._sock.settimeout(20)
        self._sock.connect(self._peer)

    def __del__(self):
        self._sock.close()

    def send(self, data):
        """Send *data* to the peer."""
        LOG.debug('Sending {0}'.format(data))
        self._sock.sendall(data.encode('ascii'))

    def recv(self):
        """Read data from the socket."""
        try:
            result = self._sock.recv(1024)
            LOG.debug('Received {0}'.format(result))
            return result
        except socket.timeout:
            LOG.warn('Reading from %s:%i timed out' % self._peer)


class Aerotech(Connection):

    """Aerotech Connection. """
    EOS_CHAR = "\n"  # string termination character
    ACK_CHAR = "%"  # acknowledge
    NAK_CHAR = "!"  # not acknowledge (wrong parameters, etc.)
    FAULT_CHAR = "#"  # task fault

    @classmethod
    def _interpret_response(cls, hle_response):
        if (hle_response[0] == Aerotech.ACK_CHAR):
            # return the data
            res = hle_response[1:
                               hle_response.index(Aerotech.EOS_CHAR)]
            LOG.debug("Interpreted response {0}.".format(res))
            return res
        if (hle_response[0] == Aerotech.NAK_CHAR):
            LOG.warn(hle_response)
            raise ValueError("Invalid command or parameter")
        if (hle_response[0] == Aerotech.FAULT_CHAR):
            raise RuntimeError("Controller task error.")

    def send(self, data):
        """Add eos special character after the command."""
        try:
            super(Aerotech, self).send(data + Aerotech.EOS_CHAR)
        except socket.error as err:
            if err.errno == socket.errno.ECONNRESET:
                LOG.debug("Connection reset by peer, reconnecting...")
                self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                self._sock.settimeout(20)
                self._sock.connect(self._peer)
                # Try again.
                super(Aerotech, self).\
                    send(data.upper() + Aerotech.EOS_CHAR)

    def recv(self):
        """Return properly interpreted answer from the controller."""
        return self._interpret_response(super(Aerotech, self).recv())

    def execute(self, data):
        """Execute command and wait for response."""
        self.send(data)
        return self.recv()



"""
Cameras supported by the libuca library.
"""
import numpy as np
from concert.quantities import q
from concert.base import Parameter
from concert.devices.cameras import base


def _new_setter_wrapper(camera, name, unit=None):
    def _wrapper(value):
        if unit:
            value = value.to(unit)

        dic = {name: value}
        camera.set_properties(**dic)

    return _wrapper


def _new_getter_wrapper(camera, name, unit=None):
    def _wrapper():
        value = camera.get_property(name)

        if unit:
            return value * unit

        return value

    return _wrapper


def _create_data_array(camera):
    bits = camera.props.sensor_bitdepth
    dtype = np.uint16 if bits > 8 else np.uint8
    dims = camera.props.roi_height, camera.props.roi_width
    array = np.zeros(dims, dtype=dtype)
    return (array, array.__array_interface__['data'][0])


class Camera(base.Camera):

    """libuca-based camera.

    All properties that are exported by the underlying camera are also visible
    in :class:`UcaCamera`.

    :raises ValueError: In case camera *name* does not exist.
    """

    def __init__(self, name):
        from gi.repository import GObject, Uca

        self._manager = Uca.PluginManager()
        self._data = None
        self._array = None

        try:
            self.camera = self._manager.get_camerav(name, [])
        except:
            raise ValueError("`{0}' is not a valid camera".format(name))

        units = {
            Uca.Unit.METER: q.m,
            Uca.Unit.SECOND: q.s,
            Uca.Unit.DEGREE_CELSIUS: q.celsius,
            Uca.Unit.COUNT: q.count
        }

        parameters = []

        for prop in self.camera.props:
            getter, setter, unit = None, None, None

            uca_unit = self.camera.get_unit(prop.name)

            if uca_unit in units:
                unit = units[uca_unit]

            if prop.flags & GObject.ParamFlags.READABLE:
                getter = _new_getter_wrapper(self.camera, prop.name, unit)

            if prop.flags & GObject.ParamFlags.WRITABLE:
                setter = _new_setter_wrapper(self.camera, prop.name, unit)

            parameter = Parameter(prop.name, getter, setter, unit)
            parameters.append(parameter)

        super(Camera, self).__init__(parameters)

    def _record_real(self):
        self._array, self._data = _create_data_array(self.camera)
        self.camera.start_recording()

    def _stop_real(self):
        self.camera.stop_recording()

    def _trigger_real(self):
        self.camera.trigger()

    def _grab_real(self):
        if not self._data:
            self._array, self._data = _create_data_array(self.camera)

        self.camera.grab(self._data)
        return self._array



"""
The :mod:`.tomoalignment` module aligns rotation axis for tomography
experiments. It uses :mod:`.rotationaxis` module as a measure of rotation
axis inclination towards :math:`y`-axis.

**Algorithm**

The procedure consists of the following steps:

#. Acquire a set of images with rotated sample
#. Determine the axis inclination angles.
#. Rotate the sample according to the measured angles.

The mentioned steps are repeated until a certain minimum threshold is reached.
"""
import numpy as np
from concert.quantities import q
from concert.asynchronous import dispatcher, wait, async
from concert.processes.base import Process


class Aligner(Process):

    """Tomographic rotation axis alignment procedure."""
    # Aligned message
    AXIS_ALIGNED = "axis-aligned"

    def __init__(self, axis_measure, scanner, x_motor, z_motor=None):
        """Contructor. *axis_measure* provides axis of rotation angular
        misalignment data, *scanner* provides image sequences with
        sample rotated around axis of rotation.
        *x_motor* turns the sample around x-axis, *z_motor* is optional
        and turns the sample around z-axis
        """
        super(Aligner, self).__init__(None)
        self._axis_measure = axis_measure
        self._scanner = scanner
        self.x_motor = x_motor
        self.z_motor = z_motor

    @async
    def run(self, absolute_eps=0.1 * q.deg):
        """
        run(absolute_eps=0.1*q.deg)

        The procedure finishes when it finds the minimum angle between an
        ellipse extracted from the sample movement and respective axes or the
        found angle drops below *absolute_eps*. The axis of rotation after
        the procedure is (0,1,0), which is the direction perpendicular
        to the beam direction and the lateral direction.
        """
        # Sometimes both z-directions need to be tried out because of the
        # projection ambiguity.
        z_direction = -1

        x_last = None
        z_last = None
        z_turn_counter = 0

        while True:
            self._axis_measure.images = self._scanner.run().result()[1]
            x_angle, z_angle = self._axis_measure()

            x_better = True if self.z_motor is not None and\
                (x_last is None or np.abs(x_angle) < x_last) else False
            z_better = True if z_last is None or np.abs(z_angle) < z_last\
                else False

            if z_better:
                z_turn_counter = 0
            elif z_turn_counter < 1:
                # We might have rotated in the opposite direction because
                # of the projection ambiguity. However, that must be picked up
                # in the next iteration, so if the two consequent angles
                # are worse than the minimum, we have the result.
                z_better = True
                z_direction = -z_direction
                z_turn_counter += 1

            x_future, z_future = None, None
            if z_better and np.abs(z_angle) >= absolute_eps:
                x_future = self.x_motor.move(z_direction * z_angle)
            if x_better and np.abs(x_angle) >= absolute_eps:
                z_future = self.z_motor.move(x_angle)
            elif (np.abs(z_angle) < absolute_eps or not z_better):
                # The newly calculated angles are worse than the previous
                # ones or the absolute threshold has been reached,
                # stop iteration.
                dispatcher.send(self, self.AXIS_ALIGNED)
                break

            wait([future for future in [x_future, z_future]
                  if future is not None])

            x_last = np.abs(x_angle)
            z_last = np.abs(z_angle)



"""Dummy"""
import random
import time
from concert.base import Parameter
from concert.devices.base import Device
from concert.asynchronous import async


class DummyDevice(Device):

    """A dummy device."""

    def __init__(self):
        parameter = Parameter('value', self._get_value, self._set_value)
        super(DummyDevice, self).__init__([parameter])
        self._value = None

    def _get_value(self):
        """Get the real value."""
        return self._value

    def _set_value(self, value):
        """The real value setter."""
        self._value = value

    @async
    def do_nothing(self):
        """Do nothing."""
        pass



"""
A graph-based data processing framework is part of the `Ufo project`_. In order
to make both projects as decoupled as possible, the :class:`.UfoProcess` class
let's an ordinary Ufo task graph look like it can be parameterized. Let's
assume, we have the following task description ::

    from gi.repository import Ufo

    # Prepare the graph
    pm = Ufo.PluginManager()
    reader = pm.get_task('reader')
    writer = pm.get_task('writer')
    bp = pm.get_task('backproject')
    bp.axis_pos = 0

    graph = Ufo.TaskGraph()
    graph.read_from_file('json')
    graph.connect_nodes(reader, bp)
    graph.connect_nodes(bp, writer)

To expose Ufo node properties as Concert parameters, we have to pass the graph,
the node and the node's property name to the :class:`UfoProcess` constructor.
In the following example, we make the axis position available to the control
system ::

    from concert.processes.ufo import UfoProcess

    process = UfoProcess(graph, bp, 'axis-pos')

Now, we can use ``process`` like any other regular device, for example print
the value or scan along a "trajectory"::

    from concert.processes.base import Scanner

    scanner = Scanner(process['axis-pos'], feedback)
    x, y = scanner.run().result()

.. _Ufo project: http://ufo.kit.edu
"""
import multiprocessing
from concert.base import Parameter
from concert.processes.base import Process
from concert.asynchronous import async


class UfoProcess(Process):

    """Wraps a Ufo task graph and export selected node properties.

    *graph* must be a Ufo task graph. *node* is a node that is connected inside
    *graph* *prop_name* is a property name of *node* and used to expose this
    property.

    :meth:`run` executes *graph* with its own scheduler instance. Use the
    *config* parameter to pass a UfoConfiguration to the scheduler.
    """

    def __init__(self, graph, node, prop_name, config=None):
        self._graph = graph
        self._config = config

        def _create_getter(node, param):
            def _wrapper():
                value = node.get_property(param)
                return value
            return _wrapper

        def _create_setter(node, param):
            def _wrapper(value):
                node.set_property(param, value)
                return self.run()
            return _wrapper

        def _create_parameter(node, prop):
            from gi.repository import GObject

            getter, setter = None, None

            if prop.flags & GObject.ParamFlags.READABLE:
                getter = _create_getter(node, prop.name)

            if prop.flags & GObject.ParamFlags.WRITABLE:
                setter = _create_setter(node, prop.name)

            param = Parameter(prop.name, getter, setter, doc=prop.blurb)
            return param

        prop = [p for p in node.props if p.name == prop_name]

        if prop:
            parameter = _create_parameter(node, prop[0])
            super(UfoProcess, self).__init__([parameter])
        else:
            msg = "Parameter {0} not in {1}".format(prop_name, node)
            raise ValueError(msg)

    @async
    def run(self):
        """Execute the graph."""
        def target():
            """Configure and run."""
            from gi.repository import Ufo

            if self._config:
                sched = Ufo.Scheduler(config=self._config)
            else:
                sched = Ufo.Scheduler()

            sched.run(self._graph)

        proc = multiprocessing.Process(target=target)
        proc.start()
        proc.join()



"""Storage Ring Device"""
from concert.quantities import q
from concert.base import Parameter
from concert.devices.base import Device


class StorageRing(Device):

    """Read-only access to storage ring information.

    .. py:attribute:: current

        Ring current

    .. py:attribute:: energy

        Ring energy

    .. py:attribute:: lifetime

        Ring lifetime in hours
    """

    def __init__(self):
        params = [Parameter('current',
                            fget=self._get_current,
                            unit=q.mA,
                            doc="Current of the ring",
                            owner_only=True),
                  Parameter('energy',
                            fget=self._get_energy,
                            unit=q.MeV,
                            doc="Energy of the ring",
                            owner_only=True),
                  Parameter('lifetime',
                            fget=self._get_lifetime,
                            unit=q.h,
                            doc="Lifetime of the ring",
                            owner_only=True)]

        super(StorageRing, self).__init__(params)

    def _get_current(self):
        raise NotImplementedError

    def _get_energy(self):
        raise NotImplementedError

    def _get_lifetime(self):
        raise NotImplementedError



"""Tomo Table"""
from concert.devices.io.base import IO, Port
from concert.connections.tango import TopoTomo


class TomoTable(IO):

    """Tomo table's I/O board at the TopoTomo beam line at ANKA."""

    def __init__(self):
        ports = [Port((6, 1), "input", self.read_port, None,
                      "Trigger input to the table."),
                 Port(6, "output", None, self.write_port,
                      "Trigger output from the table.")]
        self._device = TopoTomo().get_device("iss/toto/modbusTomotable")
        super(TomoTable, self).__init__(ports)

    def read_port(self, port):
        return self._device.ReadBits(port)

    def write_port(self, port, value):
        self._device.WriteBits((port, int(bool(value))))



"""Aerotech Controller."""
import logbook
from concert.devices.base import Device
from concert.connections.inet import Aerotech
from concert.asynchronous import async
from concert.devices.io.aerotech import IO


LOG = logbook.Logger(__name__)


class HLe(Device):

    """Aerotech Ensemble HLe controller."""
    HOST = "192.168.18.19"
    PORT = 8001

    def __init__(self):
        self._connection = Aerotech(HLe.HOST, HLe.PORT)
        super(HLe, self).__init__()
        self.ioo = IO(self._connection)

    def reset(self):
        """Reset the controller."""
        # TODO: timeout here?
        linked = False
        self._connection.execute("RESET")

        while not linked:
            try:
                self._connection = Aerotech(HLe.HOST, HLe.PORT)
            except:
                pass
            else:
                linked = True

    @async
    def get_positions(self):
        """Get positions stored on the controller's file system."""
        self.program_run(1, "position_readout.bcx")
        positions = []

        conn = Aerotech(HLe.HOST, 8000)

        while True:
            data = conn.execute("NEXT")
            if data == "ERROR":
                msg = "Error reading positions."
                LOG.error(msg)
                raise RuntimeError(msg)
            elif data == "EOF":
                break
            positions.append(float(data))

        return positions

    def program_run(self, task, program_name):
        """Execute *program_name* on task number *task*."""
        self._connection.execute("TASKRUN %d, \"%s\"" %
                                 (task, program_name))

    def program_stop(self, task):
        """Stop program execution on task number *task*."""
        self._connection.execute("PROGRAM STOP %d" % (task))

    def get_integer_register(self, register):
        """Get value stored in integer *register* on the controller."""
        self._connection.execute("IGLOBAL(%d)" % (register))

    def set_integer_register(self, register, value):
        """Set *value* stored in integer *register* on the controller."""
        self._connection.execute("IGLOBAL(%d)=%f" % (register, value))

    def get_double_register(self, register):
        """Get value stored in double *register* on the controller."""
        self._connection.execute("DGLOBAL(%d)" % (register))

    def set_double_register(self, register, value):
        """Set *value* stored in double *register* on the controller."""
        self._connection.execute("DGLOBAL(%d)=%f" % (register, value))



# -*- coding: utf-8 -*-
"""
A *device* is a software abstraction for a piece of hardware that can be
controlled.

Each device consists of a set of named :class:`Parameter` instances and
device-specific methods. If you know the parameter name, you can get
a reference to the parameter object by using the index operator::

    pos_parameter = motor['position']

To set and get parameters explicitly , you can use the :meth:`Parameter.get`
and :meth:`Parameter.set` methods::

    pos_parameter.set(1 * q.mm)
    print (pos_parameter.get().result())

Both methods will return a *Future*. A future is a promise that a result will
be delivered when asked for. In the mean time other things can and should
happen concurrently. As you can see, to get the result of a future you call its
``result()`` method.

An easier way to set and get parameter values are properties via the
dot-name-notation::

    motor.position = 1 * q.mm
    print (motor.position)

As you can see, accessing parameters this way will *always be synchronous* and
*block* execution until the value is set or fetched.

Parameter objects are not only used to communicate with a device but also carry
meta data information about the parameter. The most important ones are
:attr:`Parameter.name`, :attr:`Parameter.unit` and :attr:`Parameter.limiter` as
well as the doc string describing the parameter. Moreover, parameters can be
queried for access rights using :meth:`Parameter.is_readable` and
:meth:`Parameter.is_writable`.

To get all parameters of an object, you can iterate over the device itself ::

    for param in motor:
        print("{0} => {1}".format(param.unit, param.name))
"""
import re
from logbook import Logger
from concert.asynchronous import dispatcher
from concert.ui import get_default_table
from concert.asynchronous import async


LOG = Logger(__name__)


class UnitError(ValueError):

    """Raised when an operation is passed value with an incompatible unit."""
    pass


class LimitError(Exception):

    """Raised when an operation is passed a value that exceeds a limit."""
    pass


class HardlimitError(LimitError):

    """Raised when a hard limit is hit on the device."""
    pass


class ParameterError(Exception):

    """Raised when a parameter is accessed that does not exists."""

    def __init__(self, parameter):
        msg = "`{0}' is not a parameter".format(parameter)
        super(ParameterError, self).__init__(msg)


class ReadAccessError(Exception):

    """Raised when user tries to change a parameter that cannot be written."""

    def __init__(self, parameter):
        LOG.warn("Invalid read access on {0}".format(parameter))
        msg = "parameter `{0}' cannot be read".format(parameter)
        super(ReadAccessError, self).__init__(msg)


class WriteAccessError(Exception):

    """Raised when user tries to read a parameter that cannot be read."""

    def __init__(self, parameter):
        LOG.warn("Invalid write access on {0}".format(parameter))
        msg = "parameter `{0}' cannot be written".format(parameter)
        super(WriteAccessError, self).__init__(msg)


class MultiContext(object):

    """Multi context manager to be used in a Python `with` management.

    For example, to use multiple axes safely in one process, all you have to do
    is ::

        with MultiContext([axis1, axis2]):
            axis1.set_position()
            axis2.set_position()

    Original code by João S. O. Bueno licensed under CC-BY-3.0.
    """

    def __init__(self, *args):
        if (len(args) == 1 and
               (hasattr(args[0], "__len__") or
                hasattr(args[0], "__iter__"))):
            self.objs = list(args[0])
        else:
            self.objs = args

    def __enter__(self):
        return tuple(obj.__enter__() for obj in self.objs)

    def __exit__(self, type_, value, traceback):
        return all([obj.__exit__(type_, value, traceback)
                    for obj in self.objs])


def value_compatible(value, unit):
    """Check if a *value* of a quantity is compatible with *unit* and return
    ``True``."""
    try:
        _ = unit + value
        return True
    except ValueError:
        return False


def parameter_name_valid(name):
    """Check if a parameter *name* is correct and return ``True`` if so."""
    expr = r'^[a-zA-Z]+[a-zA-Z0-9_-]*$'
    return re.match(expr, name) is not None


class Parameter(object):

    """
    A parameter with a *name* and an optional *unit* and *limiter*.

    .. py:attribute:: name

        The name of the parameter.

    .. py:attribute:: unit

        The unit that is expected when setting a value and that is returned. If
        a unit is not compatible, a :class:`.UnitError` will be raised.

    .. py:attribute:: limiter

        A callable that receives the value and returns True or False, depending
        if the value is out of limits or not.
    """

    CHANGED = 'changed'

    def __init__(self, name, fget=None, fset=None,
                 unit=None, limiter=None,
                 doc=None, owner_only=False):

        if not parameter_name_valid(name):
            raise ValueError('{0} is not a valid parameter name'.format(name))

        self.name = name
        self.unit = unit
        self.limiter = limiter
        self.owner = None
        self._owner_only = owner_only
        self._fset = fset
        self._fget = fget
        self._value = None
        self.__doc__ = doc

    def __lt__(self, other):
        return str(self) <= str(other)

    @async
    def get(self):
        """
        get()
        Read and return the current value.

        If the parameter cannot be read, :class:`.ReadAccessError` is raised.
        """
        if not self.is_readable():
            raise ReadAccessError(self.name)

        if self._is_simple():
            return self._value

        return self._fget()

    @async
    def set(self, value, owner=None):
        """
        set(value, owner=None)
        Write *value*.

        If the parameter cannot be written, :class:`.WriteAccessError` is
        raised. If :attr:`unit` is set and not compatible with *value*,
        :class:`.UnitError` is raised.

        Once the value has been written on the device, all associated
        callbacks are called and a message is placed on the dispatcher bus.
        """
        if self._owner_only and owner != self.owner:
            raise WriteAccessError(self.name)

        if not self.is_writable():
            raise WriteAccessError(self.name)

        if self.unit and not value_compatible(value, self.unit):
            msg = "`{0}' can only receive values of unit {1} but got {2}"
            raise UnitError(msg.format(self.name, self.unit, value))

        if self.limiter and not self.limiter(value):
            msg = "{0} for `{1}' is out of range"
            raise LimitError(msg.format(value, self.name))

        def log_access(what):
            """Log access."""
            msg = "{0}: {1} {2}='{3}'"
            name = self.owner.__class__.__name__
            LOG.info(msg.format(name, what, self.name, value))

        log_access('try')

        if self._is_simple():
            self._value = value
        else:
            self._fset(value)

        log_access('set')
        self.notify()

    def notify(self):
        """Notify that the parameter value has changed."""
        dispatcher.send(self, self.CHANGED)

    def _is_simple(self):
        return self._fget is None and self._fset is None

    def is_readable(self):
        """Return `True` if parameter can be read."""
        return self._fget is not None or self._is_simple()

    def is_writable(self):
        """Return `True` if parameter can be written."""
        return self._fset is not None or self._is_simple()


class Parameterizable(object):

    """Collection of parameters.

    A :class:`Parameterizable` is iterable and returns its parameters of type
    :class:`Parameter` ::

        for param in device:
            msg = "{0} readable={1}, writable={2}"
            print(msg.format(param.name,
                             param.is_readable(), param.is_writable())

    To access a single name parameter object, you can use the ``[]`` operator::

        param = device['position']
        print param.is_readable()

    If the parameter name does not exist, a :class:`.ParameterError` is raised.

    Each parameter value is accessible as a property. If a device has a
    position it can be read and written with::

        param.position = 0 * q.mm
        print param.position
    """

    def __init__(self, parameters=None):
        self._params = {}

        if parameters:
            for parameter in parameters:
                parameter.owner = self
                self.add_parameter(parameter)

    def __str__(self):
        table = get_default_table(["Parameter", "Value"])
        table.border = False
        readable = (param for param in self if param.is_readable())

        for param in readable:
            table.add_row([param.name, str(param.get().result())])

        return table.get_string(sortby="Parameter")

    def __repr__(self):
        return '\n'.join([super(Parameterizable, self).__repr__(),
                         str(self)])

    def __iter__(self):
        for param in sorted(self._params.values()):
            yield param

    def __getitem__(self, param):
        if param not in self._params:
            raise ParameterError(param)

        return self._params[param]

    def add_parameter(self, parameter):
        """Add *parameter*.

        This will install a property :attr:`.Parameter.name` with its getter
        set to :meth:`.Parameter.get` and the setter set to
        :meth:`.Parameter.get`.  The name will only consist of underscores.

        Furthermore, two functions `set_parameter_name` and
        `get_parameter_name` will be installed that are asynchronous.
        """
        self._params[parameter.name] = parameter
        parameter.owner = self
        underscored = parameter.name.replace('-', '_')

        def _getter(self):
            return self[parameter.name].get().result()

        def _setter(self, value):
            self[parameter.name].set(value).wait()

        setattr(self.__class__, underscored, property(_getter, _setter))

        if parameter.is_readable():
            setattr(self, 'get_%s' % underscored, self[parameter.name].get)

        if parameter.is_writable():
            setattr(self, 'set_%s' % underscored, self[parameter.name].set)



"""Handle session management.

A session is an ordinary Python module that is stored in a per-user
directory."""
import os
import imp
import inspect
from concert.ui import get_default_table
from concert.devices.base import Device
from concert.processes.base import Process


def _get_save_data_path():
    env = os.environ
    if "VIRTUAL_ENV" in env:
        env["XDG_DATA_HOME"] = os.path.join(env["VIRTUAL_ENV"], "share")

    import xdg.BaseDirectory
    return xdg.BaseDirectory.save_data_path('concert')


PATH = _get_save_data_path()
DEFAULT_LOGFILE = os.path.join(PATH, 'concert.log')


def _get_param_description_table(motor):
    field_names = ["Name", "Access", "Unit", "Description"]
    table = get_default_table(field_names)
    table.border = False
    table.header = True

    def _access(parameter):
        result = 'r' if parameter.is_readable() else ''
        result += 'w' if parameter.is_writable() else ''
        return result

    for param in motor:
        dims = param.unit.units if param.unit else None
        row = [param.name, _access(param), str(dims), inspect.getdoc(param)]
        table.add_row(row)

    return table.get_string()


def _get_param_value_table(motor):
    field_names = ["Name", "Value"]
    table = get_default_table(field_names)
    table.border = False
    table.header = False

    for param in motor:
        if param.is_readable():
            table.add_row([param.name, str(param.get().result())])

    return table.get_string()


def _current_instances(instance_type):
    # Get the second frame to skip caller who called us
    frame = inspect.stack()[2]
    instances = frame[0].f_globals

    return ((name, obj) for (name, obj)
            in instances.items()
            if isinstance(obj, instance_type))


def dstate():
    """Render device state in a table."""
    field_names = ["Name", "Parameters"]
    table = get_default_table(field_names)

    for name, device in _current_instances(Device):
        values = _get_param_value_table(device)
        table.add_row([name, values])

    print(table.get_string())


def ddoc():
    """Render device documentation."""
    field_names = ["Name", "Description", "Parameters"]
    table = get_default_table(field_names)

    for name, device in _current_instances(Device):
        doc = _get_param_description_table(device)
        table.add_row([name, inspect.getdoc(device), doc])

    print(table.get_string())


def pdoc():
    """Render process documentation."""
    field_names = ["Name", "Description"]
    table = get_default_table(field_names)

    for name, process in _current_instances(Process):
        table.add_row([name, inspect.getdoc(process)])

    print(table.get_string())


def path(session):
    """Get absolute path of *session* module."""
    return os.path.join(PATH, session + '.py')


def create(session, imports=()):
    """Create a template with *session* name and write it.

    For each name in *imports* try to load it and insert `from
    concert.processes.name import *` into the session file.

    .. note:: This will *always* overwrite session.
    """
    template = 'from concert.quantities import q\n'
    template += '\n'
    template += 'from concert.session import ddoc, dstate, pdoc\n'
    template += '\n'
    template += '__doc__ = "This is session %s"\n' % session

    def _module_exists(module_name):
        try:
            __import__(module_name)
            return True
        except ImportError:
            return False

    for module in imports:
        module_name = 'concert.processes.{0}'.format(module)

        if _module_exists(module_name):
            template += 'from {0} import *'.format(module_name)
        else:
            print("{0} not found.".format(module_name))

    if not os.path.exists(PATH):
        os.mkdir(PATH)

    with open(path(session), 'w') as session_file:
        session_file.write(template)


def remove(session):
    """Remove a *session*."""
    if exists(session):
        os.unlink(path(session))


def load(session):
    """Load *session* and return the module."""
    return imp.load_source('m', path(session))


def get_existing():
    """Get all existing session names."""
    sessions = [f for f in os.listdir(PATH) if f.endswith('.py')]
    return [os.path.splitext(f)[0] for f in sessions]


def exists(session):
    """Check if *session* already exists."""
    return os.access(path(session), os.R_OK)



"""Aerotech"""
from concert.devices.io import base
from concert.devices.io.base import Port


class IO(base.IO):

    """IO class implementation."""

    def __init__(self, connection):
        ports = [Port((1, 0), "read", self.read_port, None, "Read port."),
                 Port((1, 1), "write", None, self.write_port, "Write port.")]
        super(IO, self).__init__(ports)
        self._connection = connection

    def read_port(self, port):
        return self._connection.execute("DIN(X,%d,%d)" % (port[0], port[1]))

    def write_port(self, port, value):
        self._connection.execute("DOUT(X,%d,%d:%d)" %
                                 (port[0], port[1], value))



import unittest
from concert.devices.shutters.dummy import Shutter as DummyShutter


class TestDummyShutter(unittest.TestCase):

    def setUp(self):
        self.shutter = DummyShutter()

    def test_open(self):
        self.shutter.open().wait()
        self.assertEquals(self.shutter.state, self.shutter.OPEN)

    def test_close(self):
        self.shutter.close().wait()
        self.assertEquals(self.shutter.state, self.shutter.CLOSED)



"""Dummy IO"""
from concert.devices.io import base
from concert.devices.io.base import Port


class IO(base.IO):

    """Dummy I/O device implementation."""

    def __init__(self):
        ports = [Port(0, "busy", self.read_port, None, "Camera busy flag."),
                 Port(1, "exposure", self.read_port,
                      None, "Camera exposure flag."),
                 Port(2, "acq_enable", None, self.write_port,
                      "Acquisition enable flag.")]
        super(IO, self).__init__(ports)

        self._ports = dict((port.port_id, 0)
                           for port in self if isinstance(port, Port))

        self._ports[self["busy"].port_id] = 1

    def read_port(self, port):
        return self._ports[port]

    def write_port(self, port, value):
        self._ports[port] = int(bool(value))



"""Topotomo"""
from concert.devices.shutters import base
from concert.connections.tango import TopoTomo
import time


class Shutter(base.Shutter):

    """Shutter class implementation."""

    def __init__(self, index):
        if index < 0 or index > 2:
            raise ValueError("Index must be in range [0-2].")

        super(Shutter, self).__init__()
        self._device = TopoTomo().get_device("iss/toto/rato_toto")
        self._index = index

    @property
    def index(self):
        """Return Index."""
        return self._index

    def _open(self):
        self._device.DoSPECTextCommand("shopen %d" % (self.index))
        time.sleep(5)
        self._set_state(self.OPEN)

    def _close(self):
        self._device.DoSPECTextCommand("shclose %d" % (self.index))
        time.sleep(5)
        self._set_state(self.CLOSED)



"""
The module determines a 3D circle normal inclination angles towards the
:math:`y`-axis (0,1,0) and the center of the circle.

**Algorithm**

We exploit the fact that the projected circle is an ellipse. Thus, ellipse's
major axis is the radius of rotation. If we set the beam direction to be
the :math:`z`-axis :math:`(0,0,1)`, we can determine the :math:`xy` angle
:math:`(\phi)` by the major axis angle of the ellipse. Furthermore, based
on the ratio between minor and major axis, we can determine the projected
circle's inclination from its normal, yielding the :math:`zy` angle
:math:`(\psi)` of the circle. One then needs to rotate the the sample
:math:`(-\phi,\pm \psi)`. The :math:`\psi` angle ambiguity comes from
the fact that we are working with projected data.

There are three possible situations with the ellipse fitting algorithm:

* The sample does not move in the image sequence, it is not off-centered
    enough.
* The ellipse degenerates to a line, which means the circle is parallel
    to the beam, thus we only need to rotate by :math:`-\phi`.
* The projections lead to a non-degenerate ellipse. In this case we need to
    rotate around both angles.

**Input**

An image sequence of at least 5 images with different sample positions
around the axis of rotation. It is highly recommended to use more images
in order to obtain more precise results from noisy data.

**Output**

:math:`\phi` and :math:`\psi` angles.

**Steps**

The measuring procedure is done in the following steps:

#. Sample extraction.
#. Sample tip determination (largest Euclidean distance from the sample
    entering to the image and its boundaries).
#. Ellipse fitting to the set of tip points.
#. Determine :math:`\phi` and :math:`\psi` from the projected circle.

**Asumptions**

The procedure assumes a high-absorbing sample with conical shape. Thanks to
such sample the segmentation procedure can be simplified to a thresholding
technique.

"""
import numpy as np
from scipy import ndimage
from concert.quantities import q


class Ellipse(object):

    """Ellipse fitting from a set of data points."""

    def __init__(self, images=None):
        self._images = images
        # XY angle.
        self._phi = None
        # ZY angle.
        self._psi = None
        # Ellipse center.
        self._center = None
        # Ellipse parameters.
        self._params = None
        # Sample tips
        self._tips = None

    def __call__(self):
        if self._images is None:
            raise RuntimeError("Images have not been set.")
        if self._phi is None or self._psi is None:
            self._determine_angles()
        return self._phi, self._psi

    @property
    def images(self):
        """Images of the rotated sample."""
        return self._images

    @images.setter
    def images(self, images):
        """Set image sequence to *images*."""
        self._phi = None
        self._psi = None
        self._center = None
        self._params = None
        self._tips = None
        self._images = images

    @property
    def center(self):
        """Ellipse center (y, x)."""
        if self._center is None:
            self._determine_center()

        return self._center

    @property
    def xy_angle(self):
        """Angle between x and y axes :math:`(\phi)`."""
        if self._phi is None:
            self._determine_angles()
        return self._phi

    @property
    def zy_angle(self):
        """Angle between z and y axes :math:`(\psi)`."""
        if self._psi is None:
            self._determine_angles()
        return self._psi

    @property
    def points(self):
        """Return sample tip points."""
        if self._tips is None:
            self._determine_tips()

        return self._tips

    def _fit_ellipse(self):
        """Ellipse fitting based on *points* and singular value
        decomposition.
        """
        if self._tips is None:
            self._determine_tips()

        a_matrix = _construct_matrix(self._tips)
        v_mat = np.linalg.svd(a_matrix)[2]
        self._params = v_mat.T[:, -1]

    def _determine_tips(self):
        """Get sample tips from images."""
        if self._tips is None:
            images = [_segment(image) for image in self._images]
            self._tips = _get_ellipse_points(images)
            if len(self._tips) == 0:
                raise ValueError("No sample tip points found.")

    def _determine_center(self):
        """Determine the center of the ellipse from its parameters."""
        if self._params is None:
            self._fit_ellipse()

        a_33 = np.array([[self._params[0], self._params[1] / 2],
                        [self._params[1] / 2, self._params[2]]])
        if np.linalg.det(a_33) > 0:
            x_pos = (self._params[1] * self._params[4] -
                     2 * self._params[3] * self._params[2]) /\
                (4 * self._params[0] * self._params[2] - self._params[1] ** 2)
            y_pos = (self._params[3] * self._params[1] -
                     2 * self._params[0] * self._params[4]) /\
                (4 * self._params[0] * self._params[2] - self._params[1] ** 2)
        else:
            # We are not dealing with an ellipse, use center of mass.
            y_pos, x_pos = center_of_mass(self._tips)

        # +1 for image edge-clipping compensation.
        self._center = y_pos + 1, x_pos + 1

    def _determine_angles(self):
        """
        Determine the :math:`xy` :math:`(\phi)` and :math:`zy` angle
        :math:`(\psi)` of the normal of the circle defined by the axis
        of rotation. The axis is determined by fitting an ellipse
        to the sample rotation visible by different tips positions
        in images. Return a tuple :math:`(\phi`, :math:`\psi)`.
        """
        if self._params is None:
            self._fit_ellipse()

        y_ind, x_ind = zip(*self._tips)
        x_ind = np.array(x_ind)
        y_ind = np.array(y_ind)
        d_x = x_ind.max() - x_ind.min()
        if d_x < 10:
            raise ValueError("Sample off-centering too small, " +
                             "enlarge rotation radius.")

        a_33 = np.array([[self._params[0], self._params[1] / 2],
                         [self._params[1] / 2, self._params[2]]])

        usv = np.linalg.svd(a_33)
        s_vec = usv[1]
        v_mat = usv[2]

        if np.linalg.det(a_33) <= 0:
            # Not an ellipse
            d_y = float(y_ind.max() - y_ind.min())
            angle = np.arctan(d_y / d_x) * q.rad
            if x_ind[y_ind.argmax()] <= x_ind[y_ind.argmin()]:
                # More than pi/4.
                angle = -angle
            self._phi, self._psi = angle, 0.0 * q.rad
        else:
            self._phi, self._psi = \
                np.arctan(v_mat[1][1] / v_mat[1][0]) * q.rad, \
                np.arcsin(np.sqrt(s_vec[1]) / np.sqrt(s_vec[0])) * q.rad


def _construct_matrix(points):
    """Construct a conical section matrix from data *points*."""
    y_ind, x_ind = zip(*points)
    matrix = np.empty((len(x_ind), 6), dtype=np.int)
    for i in range(len(x_ind)):
        matrix[i] = np.array([x_ind[i] ** 2, x_ind[i] * y_ind[i],
                              y_ind[i] ** 2, x_ind[i], y_ind[i], 1])

    return matrix


def _segment(image, k=3.0):
    """Segment an *image* into a sample and a background. Suppose the low
    intensities caused by a highly absorbing sample are located below 1/*k*
    of the maximum intensity in the image.
    """
    bins = np.histogram(image, 128)[1]
    if bins[0] > bins[-1] / k:
        # Minimum intensity is too high, sample out of the FOV.
        thr = 0
    else:
        thr = (bins[-1] - bins[0]) / k + bins[0]

    image[image < thr] = 0
    image[image >= thr] = 1
    image = 1 - image
    image = ndimage.binary_fill_holes(np.cast[np.bool](image))

    # Close the image to get rid of noise-caused fuzzy segmentation.
    return ndimage.binary_closing(image) + image


def _get_boundary_coordinates(coordinates, max_val):
    """Return coordinates which reside on image edges."""
    return [coor for coor in coordinates if coor % max_val == 0]


def _is_corner_point(point, shape):
    """Test if the *point* lies in one of the image corners."""
    return (point[1] == 0 or point[1] == shape[1] - 1) and\
        (point[0] == 0 or point[0] == shape[0] - 1)


def _get_intersection_points(image):
    """Get *image* edges and sample intersection points. The *image* is
    a segmented binary image."""
    y_ind, x_ind = np.where(image != 0)
    x_low = x_ind[np.where(y_ind == 0)]
    x_high = x_ind[np.where(y_ind == image.shape[0] - 1)]
    y_low = y_ind[np.where(x_ind == 0)]
    y_high = y_ind[np.where(x_ind == image.shape[1] - 1)]

    points = []
    if len(x_low) != 0:
        points.append((0, x_low[0]))
        if x_low[-1] != x_low[0]:
            points.append((0, x_low[-1]))
    if len(x_high) != 0:
        points.append((image.shape[0] - 1, x_high[0]))
        if x_high[-1] != x_high[0]:
            points.append((image.shape[0] - 1, x_high[-1]))
    if len(y_low) != 0:
        points.append((y_low[0], 0))
        if y_low[-1] != y_low[0]:
            points.append((y_low[-1], 0))
    if len(y_high) != 0:
        points.append((y_high[0], image.shape[1] - 1))
        if y_high[-1] != y_high[0]:
            points.append((y_high[-1], image.shape[1] - 1))

    if len(points) > 2:
        # The sample is big and besides intersection points it fills some
        # corners of the image.
        res = []
        for point in points:
            if not _is_corner_point(point, image.shape):
                res.append(point)
        points = res

    return points


def _get_axis_intersection(p_1, p_2, shape):
    """Get intersections of a vector perpendicular to a vector defined by
    *p_1* and *p_2* and image edges defined by image *shape*."""
    # First check if the center lies on an edge
    if p_1[0] == p_2[0]:
        return [(p_1[0], (p_1[1] + p_2[1]) / 2)]
    elif p_1[1] == p_2[1]:
        return [((p_1[0] + p_2[0]) / 2.0, p_1[1])]

    p_x = (p_1[1] + p_2[1]) / 2.0
    p_y = (p_1[0] + p_2[0]) / 2.0
    v_y = p_1[0] - p_2[0]
    v_x = p_2[1] - p_1[1]
    height, width = shape[0] - 1, shape[1] - 1

    left = p_y - v_x * p_x / v_y, 0
    right = p_y + v_x * (width - p_x) / v_y, width
    bottom = 0, p_x - v_y * p_y / v_x
    top = height, p_x + v_y * (height - p_y) / v_x

    res = set([left, right, bottom, top])
    # Filter intersections which are out of the image bounding box.
    res = [x for x in res if 0 <= x[0] <= height and 0 <= x[1] <= width]

    return res


def _get_sample_tip(image):
    """Extract sample tip from the labeled *image*."""
    # First check if the sample tip is in the FOV. If not, there are no
    # objects or the sample splits the image in half.
    if ndimage.label(image.max() - image)[1] != 1:
        # No sample found in this image. It is either completely out of the
        # FOV or it splits the image in half, thus the tip is out of the FOV.
        return None

    p_1, p_2 = _get_intersection_points(image)
    intersections = _get_axis_intersection(p_1, p_2, image.shape)

    if len(intersections) == 0:
        return None

    p_inter = None

    # Get the intersection where a sample is present.
    for intersection in intersections:
        if image[intersection] > 0:
            p_inter = intersection
            break

    if p_inter is None:
        return None

    y_ind, x_ind = np.where(image > 0)

    # Calculate distances from the intersection point.
    distances = np.sqrt((y_ind - p_inter[0]) ** 2 + (x_ind - p_inter[1]) ** 2)

    # Most distant points are candidates for the sample tip.
    indices = np.where(distances == distances.max())[0]

    tips = [(y_ind[i], x_ind[i]) for i in indices]
    tip_center = center_of_mass(tips)

    # Now find the intersection of the object and line going through it
    # in the direction to the tip.
    x_direction = tip_center[1] - p_inter[1]
    if x_direction == 0:
        # The line is parallel with y_ind axis
        x_ind = np.ones(image.shape[1], dtype=np.int64) * p_inter[1]
        y_ind = np.arange(image.shape[0])
    else:
        y_direction = tip_center[0] - p_inter[0]
        x_ind = np.arange(image.shape[1])
        y_ind = np.cast[np.int](np.round(p_inter[0] + y_direction * (
                                         x_ind - p_inter[1]) / x_direction))

    # Cut values going beyond image boundaries.
    below = np.where(y_ind < image.shape[0])[0]
    y_ind = y_ind[below]
    x_ind = x_ind[below]
    above = np.where(0 <= y_ind)[0]
    y_ind = y_ind[above]
    x_ind = x_ind[above]

    # Drop indices at which there is no object in the image.
    nonzero = np.where(image[y_ind, x_ind] > 0)[0]
    x_ind = x_ind[nonzero]
    y_ind = y_ind[nonzero]

    distances = np.sqrt((y_ind - p_inter[0]) ** 2 + (x_ind - p_inter[1]) ** 2)
    i = distances.argmax()

    return y_ind[i], x_ind[i]


def _get_sample(image):
    """Segment the *image* to a sample and a background."""
    labels = _get_regions(image)
    labels = _get_boundary_regions(labels)
    # Cut the edges by one pixel because hole filling does not affect them.
    return _get_biggest_region(labels)[1:-1, 1:-1]


def _get_ellipse_points(images):
    """Extract ellipse points from the sample in *image*, where *image* is
    a binary image."""
    tips = []
    tip = None

    for image in images:
        labels = _get_sample(image)
        tip = _get_sample_tip(labels)
        if tip is not None:
            tips.append(tip)

    return tips


def _get_biggest_region(image):
    """Get the region with the biggest area in the *image*."""
    labels, features = ndimage.label(image)
    sizes = ndimage.sum(image, labels, range(features + 1))
    max_feature = np.argmax(sizes)
    labels[labels != max_feature] = 0

    return labels


def _get_boundary_regions(labels):
    """Extract regions from *labels* which appear on image boundaries only."""
    features = np.max(labels)

    for i in range(features + 1):
        y_ind, x_ind = np.where(labels == i)
        if len(x_ind) != 0 and len(y_ind) != 0 and x_ind.min() != 0 and\
                x_ind.max() != labels.shape[1] - 1 and y_ind.min() != 0 and\
                y_ind.max() != labels.shape[0] - 1:
            labels[labels == i] = 0

    return labels


def _get_regions(image):
    """Extract regions from the binary *image* which are not too small."""
    labels, features = ndimage.label(image)
    sizes = ndimage.sum(image, labels, range(features + 1))

    # Remove small regions.
    mask = sizes < 200
    remove_indices = mask[labels]
    labels[remove_indices] = 0

    return labels


def center_of_mass(points):
    """Find the center of mass from a set of *points*."""
    y_ind, x_ind = zip(*points)

    c_y = float(np.sum(y_ind)) / len(points)
    c_x = float(np.sum(x_ind)) / len(points)

    return c_y, c_x



"""Shutter Device."""
from concert.devices.base import Device
from concert.asynchronous import async


class Shutter(Device):

    """Shutter device class implementation."""
    OPEN = "open"
    CLOSED = "closed"

    def __init__(self):
        super(Shutter, self).__init__()
        self._states = self._states.union(set([self.OPEN, self.CLOSED]))
        self._state = self.NA

    @async
    def open(self):
        """open()

        Open the shutter."""
        self._open()

    @async
    def close(self):
        """close()

        Close the shutter."""
        self._close()

    def _open(self):
        raise NotImplementedError

    def _close(self):
        raise NotImplementedError



import unittest
import logbook
from concert.quantities import q
from concert.base import *
from testfixtures import ShouldRaise, compare


class BaseDevice(Parameterizable):

    def __init__(self):
        param = Parameter('foo', fget=self._get, fset=self._set)
        super(BaseDevice, self).__init__([param])

    def _get(self):
        raise NotImplementedError

    def _set(self, value):
        raise NotImplementedError


class FooDevice(BaseDevice):

    def __init__(self, proxy):
        super(FooDevice, self).__init__()
        self.proxy = proxy

    def _get(self):
        return self.proxy.get()

    def _set(self, value):
        self.proxy.set(value)


class Proxy(object):

    def __init__(self, default):
        self.value = default

    def get(self):
        return self.value

    def set(self, value):
        self.value = value


class TestParameterizable(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()
        proxy1 = Proxy(42)
        proxy2 = Proxy(23)
        self.foo1 = FooDevice(proxy1)
        self.foo2 = FooDevice(proxy2)

    def tearDown(self):
        self.handler.pop_application()

    def test_property_identity(self):
        self.foo1.foo = 15
        self.assertEqual(self.foo1.foo, 15)
        self.assertEqual(self.foo2.foo, 23)

    def test_func_identity(self):
        self.foo1.set_foo(15).wait()
        self.assertEqual(self.foo1.get_foo().result(), 15)
        self.assertEqual(self.foo2.get_foo().result(), 23)


class TestParameter(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_names(self):
        with ShouldRaise(ValueError):
            Parameter('1pm')

        with ShouldRaise(ValueError):
            Parameter('current position')

        Parameter('this-is-correct')
        Parameter('this_too')

    def test_read_only_parameter(self):
        def getter():
            return 0

        parameter = Parameter('foo', fget=getter)
        self.assertTrue(parameter.is_readable())
        self.assertFalse(parameter.is_writable())

        compare(parameter.get().result(), 0)

        with ShouldRaise(WriteAccessError('foo')):
            parameter.set(None).result()

    def test_write_only_parameter(self):
        def setter(value):
            pass

        parameter = Parameter('foo', fset=setter)
        self.assertTrue(parameter.is_writable())
        self.assertFalse(parameter.is_readable())

        parameter.set(None).result()

        with ShouldRaise(ReadAccessError('foo')):
            parameter.get().result()

    def test_invalid_unit(self):
        def setter(value):
            pass

        parameter = Parameter('foo', fset=setter, unit=q.mm)
        parameter.set(2 * q.mm).result()

        with ShouldRaise(UnitError):
            parameter.set(2 * q.s).result()

    def test_limiter(self):
        def setter(value):
            pass

        def limit(value):
            return value >= 0 and value <= 1

        parameter = Parameter('foo', fset=setter, limiter=limit)
        parameter.set(0).result()
        parameter.set(0.5).result()
        parameter.set(1).result()

        with ShouldRaise(LimitError):
            parameter.set(1.5).result()



"""
*Processes* are software abstractions to control devices in a more
sophisticated way than just manipulating their parameters by hand. Each process
that is defined in this module provides one :meth:`run` method that is executed
asynchronously and returns whatever is appropriate for the process.
"""

from concert.base import Parameterizable
from concert.asynchronous import async


class Process(Parameterizable):

    """Base process."""

    def __init__(self, params):
        super(Process, self).__init__(params)

    @async
    def run(self):
        """run()

        Run the process. The result depends on the actual process.
        """
        raise NotImplementedError



import logbook
from concert.optimization.base import ParameterOptimizer
from concert.base import LimitError


LOG = logbook.Logger()


class Minimizer(ParameterOptimizer):

    """Minimizer tries to minimize a function

    .. math::
        y = f(x)

    .. py:attribute:: algorithm

        An algorithm which does the optimization, it is a callable.

    .. py:attribute:: alg_args

        A tuple of arguments passed to the algorithm

    .. py:attribute:: alg_kwargs

        A dictionary of keyword arguments passed to the algorithm

    The executive optimization function is then::

        algorithm(x_guess, *alg_args, **alg_kwargs)

    where *x_guess* is the x value at which the optimizer starts. If
    *alg_args* is None, the *x_guess* is derived from the current
    parameter value, otherwise *x_guess* must be the first value in
    the *alg_args* list.
    """

    def __init__(self, param, feedback, algorithm, alg_args=None,
                 alg_kwargs=None):
        super(Minimizer, self).__init__(param, feedback)
        self.algorithm = algorithm
        self.alg_args = alg_args
        if not self.alg_args:
            self.alg_args = (param.get().result(), )
        self.alg_kwargs = alg_kwargs
        if not self.alg_kwargs:
            self.alg_kwargs = {}

    def _optimize(self):
        result = self.algorithm(self.evaluate, *self.alg_args,
                                **self.alg_kwargs)
        try:
            self.param.set(result).wait()
        except LimitError:
            LOG.debug("Limit reached.")


class Maximizer(Minimizer):

    """
    The same as the :py:class:`.Minimizer` but with changed sign of the
    feedback, that is, if the function to minimize is

    .. math::

        y = f(x)

    then the new function to maximize is

    .. math::

        y = - f(x).
    """

    def __init__(self, param, feedback, algorithm, alg_args=None,
                 alg_kwargs=None):
        super(Maximizer, self).__init__(param, lambda: -feedback(),
                                        algorithm, alg_args, alg_kwargs)



"""This module provides a simple dummy camera."""
import numpy as np
from concert.quantities import q
from concert.base import Parameter
from concert.devices.cameras import base


class Camera(base.Camera):

    """Simple camera.

    *background* can be an array-like that will be used to generate the frame
    when calling :meth:`.grab`. The final image will be the background +
    poisson depending on the currently set exposure time.

    :py:attribute:: exposure_time
    :py:attribute:: roi_width
    :py:attribute:: roi_height
    :py:attribute:: sensor_pixel_width
    :py:attribute:: sensor_pixel_height
    """

    def __init__(self, background=None):
        params = [Parameter('exposure-time', unit=q.s),
                  Parameter('roi-width'),
                  Parameter('roi-height'),
                  Parameter('sensor-pixel-width', unit=q.micrometer),
                  Parameter('sensor-pixel-height', unit=q.micrometer)]

        super(Camera, self).__init__(params)

        self.exposure_time = 1 * q.ms
        self.sensor_pixel_width = 5 * q.micrometer
        self.sensor_pixel_height = 5 * q.micrometer

        if background:
            self.roi_width = background.shape[1]
            self.roi_height = background.shape[0]
            self._background = background
        else:
            shape = (640, 480)
            self.roi_width, self.roi_height = shape
            self._background = np.ones(shape)

    def _record_real(self):
        pass

    def _stop_real(self):
        pass

    def _trigger_real(self):
        pass

    def _grab_real(self):
        time = self.exposure_time.to(q.s).magnitude

        # 1e5 is a dummy correlation between exposure time and emitted e-.
        tmp = self._background + time * 1e5
        max_value = np.iinfo(np.uint16).max
        tmp = np.random.poisson(tmp)
        # Cut values beyond the bit-depth.
        tmp[tmp > max_value] = max_value

        return np.cast[np.uint16](tmp)



"""
Each motor is associated with a :class:`~.Calibration` that maps arbitrary
real-world coordinates to devices coordinates. When a calibration is associated
with an motor, the position can be changed with :meth:`Motor.set_position` and
:meth:`Motor.move`::

    from concert.devices.base import LinearCalibration
    from concert.devices.motors.ankatango import ANKATangoDiscreteMotor

    calibration = LinearCalibration(1 / q.mm, 0 * q.mm)
    motor1 = ANKATangoDiscreteMotor(connection, calibration)

    motor.position = 2 * q.mm
    motor.move(-0.5 * q.mm)

As long as an motor is moving, :meth:`Motor.stop` will stop the motion.
"""
import logbook
from concert.quantities import q
from concert.base import HardlimitError
from concert.devices.base import Device, Parameter, LinearCalibration
from concert.asynchronous import async


LOG = logbook.Logger(__name__)


class Motor(Device):

    """Base class for everything that moves.

    A motor is used with a *calibration* that conforms to the
    :class:`~.Calibration` interface to convert between user and device units. If
    *calibration* is not given, a default :class:`~.LinearCalibration` mapping
    one step to one millimeter with zero offset is assumed.

    .. py:attribute:: position

        Motor position
    """

    STANDBY = 'standby'
    MOVING = 'moving'
    LIMIT = 'limit'

    def __init__(self, calibration=None, limiter=None):
        params = [Parameter('position',
                            self._get_calibrated_position,
                            self._set_calibrated_position,
                            q.m, limiter,
                            "Position of the motor")]

        super(Motor, self).__init__(params)

        if not calibration:
            self._calibration = LinearCalibration(1 * q.count / q.mm, 0 * q.mm)
        else:
            self._calibration = calibration

        self._states = \
            self._states.union(set([self.STANDBY, self.MOVING, self.LIMIT]))

    @async
    def move(self, delta):
        """
        move(delta)

        Move motor by *delta* user units."""
        self.position += delta

    @async
    def stop(self):
        """
        stop()

        Stop the motion."""
        self._stop()

    @async
    def home(self):
        """
        home()

        Home motor.
        """
        self._home()

    @classmethod
    def in_hard_limit(cls):
        """Return *True* if motor device is in a limit state, otherwise
        *False*."""
        return False

    def _get_calibrated_position(self):
        return self._calibration.to_user(self._get_position())

    def _set_calibrated_position(self, position):
        self._set_state(self.MOVING)
        self._set_position(self._calibration.to_steps(position))

        if self.in_hard_limit():
            self._set_state(self.LIMIT)
            raise HardlimitError("Hard limit reached")

        self._set_state(self.STANDBY)

    def _get_position(self):
        raise NotImplementedError

    def _set_position(self, position):
        raise NotImplementedError

    def _stop(self):
        raise NotImplementedError

    def _home(self):
        raise NotImplementedError


class ContinuousMotor(Motor):

    """A movable on which one can set velocity.

    This class is inherently capable of discrete movement.

    """

    def __init__(self, position_calibration, velocity_calibration,
                 position_limiter=None, velocity_limiter=None):
        super(ContinuousMotor, self).__init__(position_calibration,
                                              position_limiter)

        param = Parameter('velocity',
                          self._get_calibrated_velocity,
                          self._set_calibrated_velocity,
                          q.m / q.s, velocity_limiter,
                          "Velocity of the motor")

        self.add_parameter(param)
        self._velocity_calibration = velocity_calibration

    def _get_calibrated_velocity(self):
        return self._velocity_calibration.to_user(self._get_velocity())

    def _set_calibrated_velocity(self, velocity):
        self._set_velocity(self._velocity_calibration.to_steps(velocity))

    def _get_velocity(self):
        raise NotImplementedError

    def _set_velocity(self, velocity):
        raise NotImplementedError


class MotorMessage(object):

    """Motor message."""
    POSITION_LIMIT = "position_limit"
    VELOCITY_LIMIT = "velocity_limit"



import unittest
import logbook
import time
from concert.tests import VisitChecker
from concert.asynchronous import Dispatcher

SLEEP_TIME = 0.0005


class TestDispatcher(unittest.TestCase):

    def setUp(self):
        self.dispatcher = Dispatcher()
        self.checker = VisitChecker()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_subscription(self):
        self.dispatcher.subscribe(self, 'foo', self.checker.visit)
        self.dispatcher.send(self, 'foo')
        time.sleep(SLEEP_TIME)
        self.assertTrue(self.checker.visited)

    def test_unsubscription(self):
        self.dispatcher.subscribe(self, 'foo', self.checker.visit)
        self.dispatcher.unsubscribe(self, 'foo', self.checker.visit)
        self.dispatcher.send(self, 'foo')
        time.sleep(SLEEP_TIME)
        self.assertFalse(self.checker.visited)



"""Camera-related feedbacks to be used with a :class:`.Scanner` process.

To evaluate the mean according to different exposure times you can setup a
process like this ::

    camera = Camera()
    feedback = FrameMean(camera)
    scanner = Scanner(camera['exposure-time'], feedback)
    scanner.minimum = 1 * q.ms
    scanner.maximum = 1 * q.s

    x, y = scanner.run().result()
"""
import numpy as np
from concert.quantities import q
from concert.base import Parameter


class FrameMean(Parameter):

    """Grab a frame and calculate the mean.

    *camera* is a camera object that supports :meth:`.grab`, *param_name*
    is a parameter name of *camera* that should be varied according.
    """

    def __init__(self, camera):
        self.camera = camera
        super(FrameMean, self).__init__('frame-mean', unit=q.counts)

    def __call__(self):
        frame = self.camera.grab()

        # Should this be over whole frame or just those pixels above 0?
        return np.mean(frame)


class PhotonTransfer(object):

    """Calculate the photon transfer according to procedure described by M.
    Caselle and F. Beckmann.

    *camera* is a camera object that supports the :meth:`.grab`, *dark_frame*
    is an array-like with the same dimensions as the camera frames.
    """

    def __init__(self, camera, dark_frame):
        self.camera = camera
        self.dark_frame = dark_frame

    def __call__(self):
        frame = self.camera.grab()
        return np.log(np.sum(np.abs(frame - self.dark_frame)))



"""
ANKA storage ring access via TANGO.
"""
from concert.quantities import q
from concert.devices.storagerings.base import StorageRing as BaseStorageRing
from concert.connections.tango import TopoTomo


class StorageRing(BaseStorageRing):

    """Storage Ring class implementation."""

    def __init__(self):
        super(StorageRing, self).__init__()
        # TODO: find non-beam line specific storage ring device
        self._device = TopoTomo().get_device("iss/pvss/ANKAStatus")

    def _get_current(self):
        return self._device.ECurrent * q.mA

    def _get_energy(self):
        return self._device.Energy * q.MeV

    def _get_lifetime(self):
        return self._device.Lifetime * q.hour



import time
import unittest
import logbook
from concert.quantities import q
from testfixtures import compare
from concert.tests import assert_almost_equal
from concert.devices.motors.dummy import Motor as DummyMotor
from concert.devices.cameras.dummy import Camera
from concert.processes.scan import Scanner, StepTomoScanner, ascan, dscan
from concert.tests import slow


def compare_sequences(first_sequence, second_sequence, assertion):
    for x, y in zip(first_sequence, second_sequence):
        assertion(x, y)


class TestScan(unittest.TestCase):

    def setUp(self):
        self.motor = DummyMotor()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def handle_scan(self, parameters):
        self.positions.append(parameters[0].get().result())

    def test_ascan(self):
        self.positions = []

        ascan([(self.motor['position'], -2 * q.mm, 10 * q.mm)],
              n_intervals=4,
              handler=self.handle_scan)

        expected = [-2 * q.mm, 1 * q.mm, 4 * q.mm, 7 * q.mm, 10 * q.mm]
        compare_sequences(self.positions, expected, assert_almost_equal)

    def test_dscan(self):
        self.positions = []

        self.motor.position = 2 * q.mm
        dscan([(self.motor['position'], 2 * q.mm, 10 * q.mm)],
              n_intervals=4,
              handler=self.handle_scan)

        expected = [4 * q.mm, 6 * q.mm, 8 * q.mm, 10 * q.mm, 12 * q.mm]
        compare_sequences(self.positions, expected, assert_almost_equal)

    def test_process(self):
        def feedback():
            return self.motor.position

        scanner = Scanner(self.motor['position'], feedback)
        scanner.minimum = 1 * q.mm
        scanner.maximum = 10 * q.mm
        scanner.intervals = 10
        x, y = scanner.run().result()
        compare_sequences(x, y, self.assertEqual)

    @slow
    def test_tomo_scanner(self):
        class Stage(object):

            def __init__(self):
                self.angles = []
                self.angle = 0.0 * q.deg

            def move(self, value):
                self.angle += value
                self.angles.append(self.angle)

        class PrepareChecker(object):

            def __init__(self):
                self.prepared = []

            def prepare_dark_scan(self):
                self.prepared.append('dark')

            def prepare_flat_scan(self):
                self.prepared.append('flat')

            def prepare_proj_scan(self):
                self.prepared.append('proj')

        camera = Camera()
        stage = Stage()
        checker = PrepareChecker()
        scanner = StepTomoScanner(camera, stage,
                                  checker.prepare_dark_scan,
                                  checker.prepare_flat_scan,
                                  checker.prepare_proj_scan)

        scanner.angle = 0.5 * q.deg

        darks, flats, projections = scanner.run().result()
        self.assertEqual(len(projections), len(stage.angles))
        compare(checker.prepared, ['dark', 'flat', 'proj'])



'''
Monochromator module. The device implementation needs to provide getters and
setters for wither wavelength or energy, it does not matter which one. The
conversion is handled in the base class.
'''
from concert.quantities import q
from concert.base import Parameter
from concert.devices.base import Device

# pint supports constants and defines hbar like this, but I haven't found a way
# to get the constants ...
hbar = 6.62606957e-34 * q.J * q.s


def energy_to_wavelength(energy):
    """Convert *energy* [eV-like] to wavelength [m]."""
    res = hbar * q.c / energy
    res.ito(q.m)
    return res


def wavelength_to_energy(wavelength):
    """Convert wavelength [m-like] to energy [eV]."""
    res = hbar * q.c / wavelength
    res.ito(q.eV)
    return res


class Monochromator(Device):

    """Monochromator device which is used to filter the beam in order to
    get a very narrow energy bandwidth.

    .. py:attribute:: energy

        Monochromatic energy in electron volts.

    .. py:attribute:: wavelength

        Monochromatic wavelength in meters.
    """

    def __init__(self, calibration):
        params = [Parameter("energy", self._get_energy, self._set_energy,
                            q.eV, doc="Monochromatic energy"),
                  Parameter("wavelength", self._get_wavelength,
                            self._set_wavelength,
                            q.nanometer, doc="Monochromatic wavelength")]
        super(Monochromator, self).__init__(params)
        self._calibration = calibration

    def _get_energy(self):
        # Check which method the subclass implements, use it and handle
        # conversions if necessary.
        if self.__class__._get_energy != Monochromator._get_energy:
            return self._get_energy()
        elif self.__class__._get_wavelength != Monochromator._get_wavelength:
            return wavelength_to_energy(self._get_wavelength())
        else:
            raise NotImplementedError

    def _set_energy(self, energy):
        if self.__class__._set_energy != Monochromator._set_energy:
            self._set_energy(energy)
        elif self.__class__._set_wavelength != Monochromator._set_wavelength:
            self._set_wavelength(energy_to_wavelength(energy))
        else:
            raise NotImplementedError

    def _get_wavelength(self):
        if self.__class__._get_energy != Monochromator._get_energy:
            return energy_to_wavelength(self._get_energy())
        elif self.__class__._get_wavelength != Monochromator._get_wavelength:
            return self._get_wavelength()
        else:
            raise NotImplementedError

    def _set_wavelength(self, wavelength):
        if self.__class__._set_energy != Monochromator._set_energy:
            self._set_energy(wavelength_to_energy(wavelength))
        elif self.__class__._set_wavelength != Monochromator._set_wavelength:
            self._set_wavelength(wavelength)
        else:
            raise NotImplementedError



"""Motor Dummy."""
import random
from concert.quantities import q
from concert.devices.motors import base
from concert.devices.base import LinearCalibration


class DummyLimiter(object):

    """Dummy Limiter class implementation."""

    def __init__(self, low, high):
        self.low = low
        self.high = high

    def __call__(self, value):
        return self.low < value < self.high


class Motor(base.Motor):

    """Dummy Motor class implementation."""

    def __init__(self, calibration=None,
                 limiter=None, position=None, hard_limits=None):
        super(Motor, self).__init__(calibration, limiter)
        if hard_limits is None:
            self._hard_limits = -100, 100
        else:
            self._hard_limits = hard_limits
        if not limiter:
            self._position = random.uniform(self._hard_limits[0],
                                            self._hard_limits[1])
        else:
            self._position = position

    def in_hard_limit(self):
        return self._position < self._hard_limits[0] or not\
            self._position < self._hard_limits[1]

    def _stop_real(self):
        pass

    def _set_position(self, position):
        if position < self._hard_limits[0]:
            self._position = self._hard_limits[0]
        elif not position < self._hard_limits[1]:
            # We do this funny comparison because pint is able to compare
            # "position < something" but not the other way around. See
            # https://github.com/hgrecco/pint/issues/40
            self._position = self._hard_limits[1]
        else:
            self._position = position

    def _get_position(self):
        return self._position


class ContinuousMotor(base.ContinuousMotor):

    """Dummy ContinuousMotor class implementation."""

    def __init__(self, position_calibration, velocity_calibration):
        super(ContinuousMotor, self).__init__(position_calibration,
                                              velocity_calibration)
        self._position_hard_limits = -10, 10
        self._velocity_hard_limits = -100, 100
        self._position = 0
        self._velocity = 0

    def _stop_real(self):
        self._velocity = 0

    def _set_position(self, position):
        self._position = position
        if self._position < self._position_hard_limits[0]:
            self._position = self._position_hard_limits[0]
        elif not self._position < self._position_hard_limits[1]:
            self._position = self._position_hard_limits[1]

    def _get_position(self):
        return self._position

    def _set_velocity(self, velocity):
        self._velocity = velocity

        if self._velocity < self._velocity_hard_limits[0]:
            self._velocity = self._velocity_hard_limits[0]
            # self.send(MotorMessage.VELOCITY_LIMIT)
        elif not self._velocity < self._velocity_hard_limits[1]:
            self._velocity = self._velocity_hard_limits[1]
            # self.send(MotorMessage.VELOCITY_LIMIT)

    def _get_velocity(self):
        return self._velocity



"""
Command functions used by the ``concert`` command-line interface tool.
"""
from __future__ import print_function

import sys
import os
import subprocess
import tempfile
import shutil
import logbook
import traceback
import contextlib
import urlparse
import urllib2
import concert
from concert.base import (UnitError,
                          LimitError,
                          ParameterError,
                          ReadAccessError,
                          WriteAccessError)

ARGUMENTS = {
    'edit': {'session': {'type': str}},
    'init': {'session': {'type': str},
             '--force': {'action': 'store_true',
                         'help': "Overwrite existing sessions"},
             '--imports': {'help': "Pre-import processes",
                           'metavar': 'modules',
                           'default': ''}},
    'fetch': {'url': {'type': str,
                      'help': "Fetch a Python module and save as a session."
                              " Note: Server certificates of HTTPS requests"
                              " are NOT verified!"},
              '--force': {'action': 'store_true',
                          'help': "Overwrite existing sessions"},
              '--repo': {'action': 'store_true',
                         'help':
                         "Checkout Git repository and import all files"}},
    'log': {'session': {'type': str,
                        'nargs': '?'}},
    'start': {'session': {'type': str},
              '--logto': {'choices': ['stderr', 'file'],
                          'default': 'file'},
              '--logfile': {'type': str}},
    'rm': {'sessions': {'type': str,
                        'nargs': '+',
                        'metavar': 'session'}},
    'show': {'session': {'type': str,
                         'nargs': '?',
                         'default': None,
                         'help': "Show details"}}
}


def edit(session=None):
    """Edit a session.

    Edit the session file by launching ``$EDITOR`` with the associated Python
    module file. This file can contain any kind of Python code, but you will
    most likely just add device definitions such as this::

        from concert.devices.axes.crio import LinearAxis

        crio1 = LinearAxis(None)
    """
    _exit_if_not_exists(session)
    env = os.environ
    editor = env['EDITOR'] if 'EDITOR' in env else 'vi'
    subprocess.call([editor, concert.session.path(session)])


def _get_url(path_or_url):
    result = urlparse.urlsplit(path_or_url)

    if result.scheme:
        return path_or_url

    if not os.path.exists(path_or_url):
        sys.exit("Cannot find module `{0}'.".format(path_or_url))

    result = ('file', '', os.path.abspath(path_or_url), '', '')
    return urlparse.urlunsplit(result)


def _fetch_file(url, force):
    if not url.endswith('.py'):
        sys.exit("`{0}' is not a Python module".format(url))

    session_name = os.path.basename(url[:-3])

    if concert.session.exists(session_name) and not force:
        sys.exit("`{0}' already exists".format(session_name))

    local_url = _get_url(url)

    with contextlib.closing(urllib2.urlopen(local_url)) as data:
        save_path = os.path.join(concert.session.PATH, session_name + '.py')
        with open(save_path, 'w') as output:
            output.write(data.read())


def _fetch_repo(url, force):
    path = tempfile.mkdtemp()
    cmd = 'git clone --quiet {0} {1}'.format(url, path)
    proc = subprocess.Popen(cmd.split(), stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE)
    out, err = proc.communicate()

    if proc.returncode != 0:
        sys.exit("Could not clone {0}.".format(url))

    for filename in (x for x in os.listdir(path) if x.endswith('.py')):
        session_name = os.path.basename(filename[:-3])

        if concert.session.exists(session_name) and not force:
            print("`{0}' already exists (use --force to install"
                  " anyway)".format(session_name))
        else:
            print("Add session {0} ...".format(filename[:-3]))
            shutil.copy(os.path.join(path, filename), concert.session.PATH)

    shutil.rmtree(path)


def fetch(url, force=False, repo=False):
    """Import an existing *session*."""
    if repo:
        _fetch_repo(url, force)
    else:
        _fetch_file(url, force)


def init(session=None, imports="", force=False):
    """Create a new session.

    *Additional options*:

    .. cmdoption:: --force

        Create the session even if one already exists with this name.
    """
    if concert.session.exists(session) and not force:
        message = "Session `{0}' already exists."
        message += " Use --force to create it anyway."
        print(message.format(session))
    else:
        concert.session.create(session, imports.split())


def log(session=None):
    """Show session logs.

    If a *session* is not given, the log command shows entries from all
    sessions.
    """
    logfile = concert.session.DEFAULT_LOGFILE

    if not os.path.exists(logfile):
        return

    # This is danger zone here because we run subprocess.call with shell=True.
    # However, the only input that we input is args.session which we check
    # first and the logfile itself.

    if session:
        _exit_if_not_exists(session)
        cmd = 'grep "{0}:" {1} | less'.format(session, logfile)
    else:
        cmd = 'less {0}'.format(logfile)

    subprocess.call(cmd, shell=True)


def rm(sessions=[]):
    """Remove one or more sessions.

    .. note::

        Be careful. The session file is unlinked from the file system and no
        backup is made.

    """
    for session in sessions:
        print("Removing {0}...".format(session))
        concert.session.remove(session)


def show(session=None):
    """Show available sessions or details of a given *session*."""
    if session:
        try:
            module = concert.session.load(session)
            print(module.__doc__)
        except IOError:
            print("Cannot find {0}".format(session))
        except ImportError as exception:
            print("Cannot import {0}: {1}".format(session, exception))
    else:
        sessions = concert.session.get_existing()
        print("Available sessions:")

        for session in sessions:
            print("  %s" % session)


def start(session=None, logto='file', logfile=None):
    """Start a session.

    Load the session file and launch an IPython shell. Every definition that
    was made in the module file is available via the ``m`` variable. Moreover,
    the quantities package is already loaded and named ``q``. So, once the
    session has started you could access motors like this::

        $ concert start tomo

        This is session tomo
        Welcome to Concert 0.0.1
        In [1]: m.crio1.set_positon(2.23 * q.mm)
        In [2]: m.crio1.get_position()
        Out[2]: array(2.23) * mm

    *Additional options*:

    .. cmdoption:: --logto={stderr, file}

        Specify a method for logging events. If this flag is not specified,
        ``file`` is used and assumed to be
        ``$XDG_DATA_HOME/concert/concert.log``.

    .. cmdoption:: --logfile=<filename>

        Specify a log file if ``--logto`` is set to ``file``.

    """
    _exit_if_not_exists(session)
    handler = None

    if logto == 'file':
        filename = logfile if logfile else concert.session.DEFAULT_LOGFILE
        handler = logbook.FileHandler(filename)
    else:
        handler = logbook.StderrHandler()

    handler.format_string = '[{record.time}] {record.level_name}: \
%s: {record.channel}: {record.message}' % session

    # Add session path, so that sessions can import other sessions
    sys.path.append(concert.session.PATH)
    try:
        module = concert.session.load(session)
    except:
        traceback.print_exc()
        sys.exit(1)

    _run_shell(handler, module)


def _get_module_variables(module):
    attrs = [attr for attr in dir(module) if not attr.startswith('_')]
    return dict((attr, getattr(module, attr)) for attr in attrs)


def _run_shell(handler, module=None):
    def _handler(_shell, _etype, evalue, _traceback_, tb_offset=None):
        print("Sorry, {0}".format(str(evalue)))
        return None

    try:
        from concert.quantities import q

        print("Welcome to Concert {0}".format(concert.__version__))

        if module:
            print(module.__doc__)

        globals().update(_get_module_variables(module))

        with handler.applicationbound():
            import IPython

            if IPython.__version__ < '0.11':
                from IPython.Shell import IPShellEmbed
                shell = IPShellEmbed()
            else:
                from IPython.frontend.terminal.embed import \
                    InteractiveShellEmbed as ShellEmbed
                shell = ShellEmbed(banner1='')

                exceptions = (UnitError,
                              LimitError,
                              ParameterError,
                              ReadAccessError,
                              WriteAccessError)
                shell.set_custom_exc(exceptions, _handler)

            shell()
    except ImportError as exception:
        msg = "You must install IPython to run the Concert shell: {0}"
        print(msg.format(exception))


def _exit_if_not_exists(session):
    if not concert.session.exists(session):
        message = "Session `{0}' does not exist. Run `concert init {0}' first."
        sys.exit(message.format(session))



import unittest
import logbook
import numpy as np
from testfixtures import ShouldRaise
from concert.quantities import q
from concert.measures.rotationaxis import Ellipse
from concert.tests.util.rotationaxis import SimulationCamera
from concert.devices.base import LinearCalibration
from concert.devices.motors.dummy import Motor
from concert.tests import slow
from concert.processes.scan import Scanner


class TestRotationAxisMeasure(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()
        self.x_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.x_motor["position"].unit = q.deg
        self.y_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.y_motor["position"].unit = q.deg
        self.z_motor = Motor(LinearCalibration(q.count / q.deg, 0 * q.deg),
                             hard_limits=(-1e5, 1e5))
        self.z_motor["position"].unit = q.deg

        # The bigger the image size, the more images we need to determine
        # the center correctly.
        self.image_source = SimulationCamera(256, self.x_motor["position"],
                                             self.y_motor["position"],
                                             self.z_motor["position"])

        # A scanner which scans the rotation axis.
        self.scanner = Scanner(
            self.y_motor["position"], self.image_source.grab)
        self.scanner.minimum = 0 * q.rad
        self.scanner.maximum = 2 * np.pi * q.rad
        self.scanner.intervals = 50

        self.measure = Ellipse()

        # Allow 1 px misalignment in y-direction.
        self.eps = np.arctan(2.0 / self.image_source.rotation_radius) * q.rad

    def tearDown(self):
        self.handler.pop_application()

    def make_images(self, x_angle, z_angle):
        self.x_motor.position = z_angle
        self.z_motor.position = x_angle
        self.measure.images = self.scanner.run().result()[1]

    def align_check(self, x_angle, z_angle):
        self.make_images(x_angle, z_angle)
        phi, psi = self.measure()

        assert phi + x_angle < self.eps
        assert np.abs(psi) - np.abs(z_angle) < self.eps

    def center_check(self):
        assert np.abs(self.measure.center[1] -
                      self.image_source.ellipse_center[1]) < 1
        assert np.abs(self.measure.center[0] -
                      self.image_source.ellipse_center[0]) < 1

    @slow
    def test_out_of_fov(self):
        with ShouldRaise(ValueError("No sample tip points found.")):
            self.measure.images = np.ones((self.scanner.intervals,
                                           self.image_source.size,
                                           self.image_source.size))
            self.measure()

    @slow
    def test_center_no_rotation(self):
        self.make_images(0 * q.deg, 0 * q.deg)
        self.center_check()

    @slow
    def test_center_only_x(self):
        self.make_images(17 * q.deg, 0 * q.deg)
        self.center_check()

    @slow
    def test_center_only_z(self):
        self.make_images(0 * q.deg, 11 * q.deg)
        self.center_check()

    @slow
    def test_center_positive(self):
        self.make_images(17 * q.deg, 11 * q.deg)
        self.center_check()

    @slow
    def test_center_negative_positive(self):
        self.make_images(-17 * q.deg, 11 * q.deg)
        self.center_check()

    @slow
    def test_center_positive_negative(self):
        self.make_images(17 * q.deg, -11 * q.deg)
        self.center_check()

    @slow
    def test_center_negative(self):
        self.make_images(-17 * q.deg, -11 * q.deg)
        self.center_check()

    @slow
    def test_only_x(self):
        """Only misaligned laterally."""
        self.align_check(0 * q.deg, 0 * q.deg)

    @slow
    def test_only_z(self):
        """Only misaligned in the beam direction."""
        self.align_check(0 * q.deg, 11 * q.deg)

    @slow
    def test_huge_x(self):
        self.image_source.scale = (3, 0.25, 3)
        self.align_check(60 * q.deg, 11 * q.deg)

    @slow
    def test_huge_z(self):
        self.image_source.scale = (3, 0.25, 3)
        self.align_check(11 * q.deg, 60 * q.deg)

    @slow
    def test_positive(self):
        self.align_check(17 * q.deg, 11 * q.deg)

    @slow
    def test_negative_positive(self):
        self.align_check(-17 * q.deg, 11 * q.deg)

    @slow
    def test_positive_negative(self):
        self.align_check(17 * q.deg, -11 * q.deg)

    @slow
    def test_negative(self):
        self.align_check(-17 * q.deg, -11 * q.deg)



import unittest
import logbook
from concert.quantities import q
from concert.devices.base import LinearCalibration
from concert.devices.motors.base import Motor
from concert.devices.motors.dummy import Motor as DummyMotor,\
    ContinuousMotor as DummyContinuousMotor


def test_default_motor_has_default_calibration():
    class MockMotor(Motor):
        def __init__(self):
            self._position = 0
            super(MockMotor, self).__init__()

        def _set_position(self, position):
            self._position = position

        def _get_position(self):
            return self._position

    handler = logbook.TestHandler()
    handler.push_application()

    motor = MockMotor()
    motor.move(-1 * q.mm).wait()
    assert motor.position == -1 * q.mm
    motor.position = 2.3 * q.mm
    assert motor.position == 2.3 * q.mm

    handler.pop_application()


class TestDummyMotor(unittest.TestCase):

    def setUp(self):
        self.motor = DummyMotor()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_set_position(self):
        position = 1 * q.mm
        self.motor.position = position
        self.assertEqual(position, self.motor.position)

    def test_move(self):
        position = 1 * q.mm
        delta = 0.5 * q.mm
        self.motor.position = position
        self.motor.move(delta).wait()
        self.assertEqual(position + delta, self.motor.position)


class TestContinuousDummyMotor(unittest.TestCase):

    def setUp(self):
        position_calibration = LinearCalibration(q.count / q.mm, 0 * q.mm)
        velocity_calibration = LinearCalibration(q.count / (q.mm / q.s),
                                                 0 * (q.mm / q.s))

        self.motor = DummyContinuousMotor(position_calibration,
                                          velocity_calibration)

        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_set_velocity(self):
        velocity = 1 * q.mm / q.s
        self.motor.velocity = velocity
        self.assertEqual(velocity, self.motor.velocity)


class TestMotorCalibration(unittest.TestCase):

    def setUp(self):
        self.steps_per_mm = 10. * q.count / q.mm
        calibration = LinearCalibration(self.steps_per_mm, 0 * q.mm)

        class MockMotor(Motor):

            def __init__(self):
                super(MockMotor, self).__init__(calibration)
                self._position = 0 * q.count

            def _stop_real(self):
                pass

            def _set_position(self, position):
                self._position = position * q.count

            def _get_position(self):
                return self._position / q.count

        self.motor = MockMotor()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    def test_set_position(self):
        position = 100 * q.mm
        steps = position * self.steps_per_mm

        self.motor.position = position
        self.assertEqual(self.motor._position, steps)
        self.assertEqual(self.motor.position, position)



import unittest
import time
import random
import logbook
from concurrent.futures import Future
from testfixtures import ShouldRaise
from concert.devices.dummy import DummyDevice
from concert.asynchronous import async, wait
from concert.tests import slow
from concert import asynchronous


@async
def func():
    pass


@async
def bad_func():
    raise RuntimeError


class TestAsync(unittest.TestCase):

    def setUp(self):
        self.device = DummyDevice()
        self.handler = logbook.TestHandler()
        self.handler.push_application()

    def tearDown(self):
        self.handler.pop_application()

    @slow
    def test_wait(self):
        @async
        def long_func():
            time.sleep(random.random() / 50.)

        futs = []
        for i in range(10):
            futs.append(long_func())

        wait(futs)

        for future in futs:
            self.assertTrue(future.done(), "Not all futures finished.")

    def test_exceptions(self):
        with ShouldRaise(TypeError):
            wait([func(0)])

        with ShouldRaise(RuntimeError):
            wait([bad_func()])

    def test_is_async(self):
        self.assertTrue(asynchronous.is_async(func))

        def sync_func():
            pass

        self.assertFalse(asynchronous.is_async(sync_func))

    def test_futures(self):
        future1 = func()
        future2 = func().wait()
        self.assertEqual(future1.__class__, future2.__class__,
                         "Wait method does not return a future.")

        with ShouldRaise(TypeError):
            func(0).wait()

        with ShouldRaise(RuntimeError):
            bad_func().wait()

    def test_async_function(self):
        future = func()
        self.assertTrue(isinstance(future, Future),
                        "Function was not run asynchronously.")

    def test_async_accessors(self):
        future1 = self.device.set_value(15)
        future2 = self.device.get_value()

        self.assertTrue(isinstance(future1, Future),
                        "Setter accessor does not return a future.")
        self.assertTrue(isinstance(future2, Future),
                        "Getter accessor does not return a future.")

        wait([future1, future2])

    def test_async_parameter(self):
        future1 = self.device["value"].set(15)
        future2 = self.device["value"].get()

        self.assertTrue(isinstance(future1, Future),
                        "Setter does not return a future.")
        self.assertTrue(isinstance(future2, Future),
                        "Getter does not return a future.")

        wait([future1, future2])

    def test_async_method(self):
        future = self.device.do_nothing()

        self.assertTrue(isinstance(future, Future),
                        "Asynchronous method does not return a future.")



"""Shutter Dummy."""
from concert.devices.shutters import base


class Shutter(base.Shutter):

    """Shutter class implementation."""

    def __init__(self):
        super(Shutter, self).__init__()

    def _open(self):
        self._set_state(self.OPEN)

    def _close(self):
        self._set_state(self.CLOSED)



"""
Axes based on the CompactRIO controller.
"""
import readline
from concert.quantities import q
from concert.devices.base import LinearCalibration
from concert.devices.motors.base import Motor
from concert.connections.inet import Connection


CRIO_HOST = 'cRIO9074-Motion.ka.fzk.de'
CRIO_PORT = 6342


class LinearMotor(Motor):

    """A linear motor that moves in two directions."""

    def __init__(self):
        calibration = LinearCalibration(50000 / q.mm, -1 * q.mm)
        super(LinearMotor, self).__init__(calibration)

        self._connection = Connection(CRIO_HOST, CRIO_PORT)
        self['position'].limiter = lambda x: x >= 0 * q.mm and x <= 2 * q.mm

        self._home()
        self._steps = 0

    def _get_position(self):
        return self._steps

    def _set_position(self, steps):
        self._connection.send('lin %i\r\n' % steps)
        self._connection.recv()
        self._steps = steps

    def _stop(self):
        pass

    def _home(self):
        self._set_position(0)


class RotationMotor(Motor):

    """A rotational motor."""

    def __init__(self):
        calibration = LinearCalibration(50000 / q.mm, 0 * q.mm)
        super(RotationMotor, self).__init__(calibration)

        self._steps = 0
        self._connection = Connection(CRIO_HOST, CRIO_PORT)
        self['position'].limiter = lambda x: x >= 0 * q.mm and x <= 2 * q.mm

    def _get_position(self):
        return self._steps

    def _set_position(self, steps):
        self._steps = steps
        self._connection.send('rot %i\r\n' % steps)
        self._connection.recv()

    def _stop(self):
        pass


def main():
    """Main definition."""
    readline.parse_and_bind('tab: complete')

    linear_device = LinearMotor()
    rotation_device = RotationMotor()

    while True:
        line = raw_input('> ')
        if line == 'q':
            break

        try:
            command, value = line.split()
            if command == 'r':
                rotation_device.set_position(float(value) * q.mm)
            elif command == 'm':
                linear_device.set_position(float(value) * q.mm)
        except ValueError:
            print("Commands: `r [NUM]', `m [NUM]', `q'")


if __name__ == '__main__':
    main()



"""
Tango motors with ANKA specific interfaces.
"""
import time
import logbook
from concert.devices.motors import base

LOG = logbook.Logger(__name__)

try:
    import PyTango
except ImportError:
    LOG.warn("PyTango is not installed.")


SLEEP_TIME = 0.01
# Yes, it is really THAT slow!
# TODO: when RATO is not used reconsider, it might get faster.
SLOW_SLEEP_TIME = 1.0


class Motor(base.Motor):

    """A motor based on ANKA Tango motor interface."""

    def __init__(self, device, calibration, _position_limit=None):
        super(Motor, self).__init__(calibration)
        self._device = device

    def in_hard_limit(self):
        return self._device.BackwardLimitSwitch or \
            self._device.ForwardLimitSwitch

    def _get_state(self):
        tango_state = self._device.state()
        if tango_state == PyTango.DevState.MOVING:
            state = self.MOVING
        elif tango_state == PyTango.DevState.STANDBY:
            state = self.STANDBY
        else:
            state = self.NA

        return state

    def _set_position(self, position):
        self._device.position = position

        time.sleep(SLOW_SLEEP_TIME)

        while self._get_state() == base.Motor.MOVING:
            time.sleep(SLEEP_TIME)

    def _get_position(self):
        return self._device.position

    def _stop(self):
        self._device.Stop()

        while self._device.state() == PyTango.DevState.RUNNING:
            time.sleep(SLEEP_TIME)

    def _home(self):
        pass



""" This module provides different dummy measures."""


class DummyGradientMeasure(object):

    """Gradient measure that returns a quadratic fall-off of *parameter* from
    *max_position*."""

    def __init__(self, parameter, max_position):
        self.max_position = max_position
        self._max_gradient = 1e4
        self._param = parameter

    def __call__(self):
        value = self._param.get().result()
        position = value.to(self.max_position.units).magnitude
        return self._max_gradient - \
            (position - self.max_position.magnitude) ** 2



"""Tango Device Factory."""
import os
import PyTango


class DeviceFactory(object):

    """Tango device factory based on *host* and *port*."""

    def __init__(self, host, port):
        self._host = host
        self._port = port

    def get_device(self, uri):
        """Get Device."""
        # We need to do this all the time for case it was changed by
        # another factory settings.
        # TODO: check if there is a way to adjust the host in PyTango.
        os.environ["TANGO_HOST"] = "%s:%d" % (self._host, self._port)
        return PyTango.DeviceProxy(uri)


class TopoTomo(DeviceFactory):

    """TopoTomo beam line tango device factory."""

    def __init__(self):
        super(TopoTomo, self).__init__("anka-tango", 10018)



import unittest
from testfixtures import ShouldRaise
from concert.quantities import q
from concert.devices.base import Calibration, LinearCalibration


def test_not_implemented():
    calibration = Calibration()

    with ShouldRaise(NotImplementedError):
        value = calibration.to_user(1 * q.count)

    with ShouldRaise(NotImplementedError):
        value = calibration.to_steps(2 * q.mm)


class TestLinearCalibration(unittest.TestCase):
    STEPS_PER_UNIT = 5000
    OFFSET = 0 * q.mm

    def setUp(self):
        self.calibration = LinearCalibration(self.STEPS_PER_UNIT * q.count /
                                             q.mm,
                                             self.OFFSET)

    def test_to_user(self):
        user = self.calibration.to_user(self.STEPS_PER_UNIT)
        self.assertEqual(user, 1 * q.mm + self.OFFSET)

    def test_to_steps(self):
        steps = self.calibration.to_steps(1 * q.mm)
        self.assertEqual(steps, self.STEPS_PER_UNIT)

    def test_different_units(self):
        value = 1 * q.m
        steps = self.calibration.to_steps(value)
        str_steps = "%.1f" % steps

        self.assertEqual(str_steps, "%.1f" % (self.STEPS_PER_UNIT * 1e3))



"""Monochromator Dummy"""
from concert.quantities import q
from concert.devices.monochromators import base


class Monochromator(base.Monochromator):

    """Monochromator class implementation."""

    def __init__(self, calibration):
        super(Monochromator, self).__init__(calibration)
        self._energy = 100 * q.keV

    def _get_energy(self):
        return self._energy

    def _set_energy(self, energy):
        self._energy = energy



import unittest
import logbook
import scipy
import numpy as np
from scipy.ndimage import fourier
from concert.tests import assert_almost_equal
from concert.quantities import q
from concert.processes.alignment import focus
from concert.devices.cameras.base import Camera
from concert.devices.motors.dummy import Motor


MIN_POSITION = 0 * q.mm
MAX_POSITION = 100 * q.mm
FOCUS_POSITION = 35 * q.mm


class BlurringCamera(Camera):
    def __init__(self, motor):
        self._original = scipy.misc.lena()
        self.motor = motor

    def _grab_real(self):
        sigma = abs((self.motor.position - FOCUS_POSITION).magnitude)
        return fourier.fourier_gaussian(self._original, sigma)

    def _record_real(self):
        pass

    def _stop_real(self):
        pass

    def _trigger_real(self):
        pass


def test_focusing():
    handler = logbook.TestHandler()
    handler.push_application()

    motor = Motor(hard_limits=(MIN_POSITION.magnitude,
                               MAX_POSITION.magnitude))
    motor.position = 85. * q.mm
    camera = BlurringCamera(motor)
    focus(camera, motor).wait()
    assert_almost_equal(motor.position, FOCUS_POSITION, 1e-2)

    handler.pop_application()



import unittest
import logbook
from concert.quantities import q
from concert.tests import slow
from concert.measures.dummy import DummyGradientMeasure
from concert.devices.motors.dummy import Motor as DummyMotor, DummyLimiter
from concert.optimization.optimizers import Maximizer
from concert.optimization import algorithms


class TestDummyFocusing(unittest.TestCase):

    def setUp(self):
        self.handler = logbook.TestHandler()
        self.handler.push_application()
        self._motor = DummyMotor()
        self._motor.position = 0 * q.mm
        self._feedback = DummyGradientMeasure(
            self._motor['position'], 18.75 * q.mm)
        self.epsilon = 1 * q.um
        self.focuser = Maximizer(self._motor["position"], self._feedback,
                                 algorithms.halver,
                                (1 * q.mm,),
                                 {"epsilon": self.epsilon,
                                  "max_iterations": 1000})
        self._position_eps = 1e-1 * q.mm
        self._gradient_cmp_eps = 1e-1

    def tearDown(self):
        self.handler.pop_application()

    def _check_position(self, position, other_position):
        lower_bound_ok = other_position - self._position_eps <= position
        upper_bound_ok = position <= other_position + self._position_eps

        self.assertTrue(lower_bound_ok and upper_bound_ok,
                        "Motor position: %s " %
                        (str(position)) +
                        "differs more than by epsilon: %s " %
                        str(self._position_eps) +
                        "from the given position: %s" % (str(other_position)))

    @slow
    def test_maximum_in_limits(self):
        self.focuser.run().wait()
        self._check_position(self._motor.position,
                             self._feedback.max_position)

    @slow
    def test_huge_step_in_limits(self):
        self.focuser.step = 1000 * q.mm
        self.focuser.run().wait()
        self._check_position(self._motor.position,
                             self._feedback.max_position)

    @slow
    def test_maximum_out_of_limits_right(self):
        self._feedback.max_position = \
            (self._motor._hard_limits[1] + 50) * q.mm

        self.focuser.run().wait()
        self._check_position(self._motor.position,
                             self._motor._hard_limits[1] * q.mm)

    @slow
    def test_maximum_out_of_limits_left(self):
        self._feedback.max_position = \
            (self._motor._hard_limits[0] - 50) * q.mm
        self.focuser.run().wait()
        self._check_position(self._motor.position,
                             self._motor._hard_limits[0] * q.mm)

    @slow
    def test_maximum_out_of_soft_limits_right(self):
        motor = DummyMotor(limiter=DummyLimiter(25 * q.mm,
                                                75 * q.mm), position=50)
        feedback = DummyGradientMeasure(motor['position'], 80 * q.mm)
        focuser = Maximizer(motor["position"], feedback, algorithms.halver,
                            (motor.position,),
                            {"initial_step": 10 * q.mm,
                             "max_iterations": 1000})
        focuser.run().wait()

        self._check_position(motor.position, 75 * q.mm)

    @slow
    def test_maximum_out_of_soft_limits_left(self):
        motor = DummyMotor(limiter=DummyLimiter(25 * q.mm,
                                                75 * q.mm),
                           position=50)
        feedback = DummyGradientMeasure(motor['position'], 20 * q.mm)
        focuser = Maximizer(motor["position"], feedback, algorithms.halver,
                           (motor.position, 10 * q.mm, self.epsilon),
                            {"max_iterations": 1000})
        focuser._optimize()  # run().wait()
        self._check_position(motor.position, 25 * q.mm)

    @slow
    def test_huge_step_out_of_limits_right(self):
        # Right.
        self._feedback.max_position = (self._motor._hard_limits[1] + 50) * q.mm

        focuser = Maximizer(self._motor["position"], self._feedback,
                            algorithms.halver,
                           (self._motor.position, 1000 * q.mm, self.epsilon),
                            {"max_iterations": 1000})
        focuser.run().wait()
        self._check_position(self._motor.position,
                             self._motor._hard_limits[1] * q.mm)

    @slow
    def test_huge_step_out_of_limits_left(self):
        focuser = Maximizer(self._motor["position"], self._feedback,
                            algorithms.halver,
                           (self._motor.position, 1000 * q.mm, self.epsilon),
                            {"max_iterations": 1000})
        self._feedback.max_position = (self._motor._hard_limits[0] - 50) * q.mm
        focuser.run().wait()
        self._check_position(self._motor.position,
                             self._motor._hard_limits[0] * q.mm)

    @slow
    def test_identical_gradients(self):
        # Some gradient level reached and then by moving to another position
        # the same level is reached again, but global gradient maximum
        # is not at this motor position.
        self._motor.position = -0.00001 * q.mm
        self.focuser.step = 10 * q.mm
        self.focuser.run().wait()
        self._check_position(self._motor.position,
                             self._feedback.max_position)



"""Prettytable"""
import prettytable


def get_default_table(field_names):
    """Return a prettytable styled for use in the shell. *field_names* is a
    list of table header strings."""
    table = prettytable.PrettyTable(field_names)
    table.border = True
    table.hrules = prettytable.ALL
    table.vertical_char = ' '
    table.junction_char = '-'
    for name in field_names:
        table.align[name] = 'l'
    return table
